@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -259,9 +259,9 @@ main(
     if (geteuid() == 0) {
 	check_running_as(RUNNING_AS_ROOT);
 	initgroups(CLIENT_LOGIN, get_client_gid());
-	setgid(get_client_gid());
-	setegid(get_client_gid());
-	seteuid(get_client_uid());
+	if(setgid(get_client_gid()) != 0) { error("Can't set gid"); };
+	if(setegid(get_client_gid()) != 0) { error("Can't set egid"); };
+	if(seteuid(get_client_uid()) != 0) { error("Can't set euid"); };
     } else {
 	check_running_as(RUNNING_AS_CLIENT_LOGIN);
     }
@@ -491,7 +491,7 @@ main(
 
     /* krb5 require the euid to be 0 */
     if (strcasecmp(auth, "krb5") == 0) {
-	seteuid((uid_t)0);
+	if(seteuid((uid_t)0) != 0) { error("Can't set euid to 0"); };
     }
 
     /*
@@ -1837,17 +1837,19 @@ service_delete(
      * bother to waitpid for it */
     assert(as->pid > 0);
     pid = waitpid(as->pid, NULL, WNOHANG);
-    if (pid != as->pid && kill(as->pid, SIGTERM) == 0) {
+    if (pid == 0 && kill(as->pid, SIGTERM) == 0) {
 	pid = waitpid(as->pid, NULL, WNOHANG);
 	count = 5;
-	while (pid != as->pid && count > 0) {
+	while (pid == 0 && count > 0) {
 	    count--;
 	    sleep(1);
 	    pid = waitpid(as->pid, NULL, WNOHANG);
 	}
-	if (pid != as->pid) {
+	if (pid == 0) {
 	    g_debug("Process %d failed to exit", (int)as->pid);
 	}
+    } else {
+	g_debug("Waitpid for process %d failed: %s", (int)as->pid, strerror(errno));
     }
 
     serviceq = g_slist_remove(serviceq, (gpointer)as);
