@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -87,6 +88,9 @@ static amregex_t init_re_table[] = {
   AM_IGNORE_RE(": Directory is new$"),
   AM_IGNORE_RE(": Directory has been renamed"),
 
+  /* GNU tar 1.27 */
+  AM_NORMAL_RE(": directory is on a different filesystem; not dumped"),
+
   /* GNU tar 1.13.17 will print this warning when (not) backing up a
      Unix named socket.  */
   AM_NORMAL_RE(": socket ignored$"),
@@ -361,7 +365,6 @@ main(
     /* parse argument */
     command = argv[1];
 
-    gnutar_listdir = stralloc(getconf_str(CNF_GNUTAR_LIST_DIR));
     argument.config     = NULL;
     argument.host       = NULL;
     argument.message    = 0;
@@ -532,6 +535,10 @@ main(
 	g_critical(_("errors processing config file"));
     }
 
+    if (!gnutar_listdir) {
+	gnutar_listdir = g_strdup(getconf_str(CNF_GNUTAR_LIST_DIR));
+    }
+
     re_table = build_re_table(init_re_table, normal_message, ignore_message,
 			      strange_message);
 
@@ -610,6 +617,9 @@ main(
 	fprintf(stderr, "Unknown command `%s'.\n", command);
 	exit (1);
     }
+
+    dbclose();
+
     return 0;
 }
 
@@ -680,7 +690,6 @@ amgtar_selfcheck(
 	printf(_("ERROR [GNUTAR program not available]\n"));
     }
 
-    set_root_privs(1);
     if (gnutar_listdir && strlen(gnutar_listdir) == 0)
 	gnutar_listdir = NULL;
     if (gnutar_listdir) {
@@ -689,6 +698,7 @@ amgtar_selfcheck(
 	printf(_("ERROR [No GNUTAR-LISTDIR]\n"));
     }
 
+    set_root_privs(1);
     if (gnutar_directory) {
 	check_dir(gnutar_directory, R_OK);
     } else if (argument->dle.device) {
@@ -1165,7 +1175,7 @@ amgtar_restore(
 	if (argument->dle.disk) {
 	    sdisk = sanitise_filename(argument->dle.disk);
 	} else {
-	    sdisk = g_strdup_printf("no_dle-%d", getpid());
+	    sdisk = g_strdup_printf("no_dle-%d", (int)getpid());
 	}
 	exclude_filename= vstralloc(AMANDA_TMPDIR, "/", "exclude-", sdisk,  NULL);
 	exclude_list = fopen(argument->dle.exclude_list->first->name, "r");
@@ -1205,7 +1215,7 @@ amgtar_restore(
 	if (argument->dle.disk) {
 	    sdisk = sanitise_filename(argument->dle.disk);
 	} else {
-	    sdisk = g_strdup_printf("no_dle-%d", getpid());
+	    sdisk = g_strdup_printf("no_dle-%d", (int)getpid());
 	}
 	include_filename = vstralloc(AMANDA_TMPDIR, "/", "include-", sdisk,  NULL);
 	include = fopen(include_filename, "w");
@@ -1434,7 +1444,12 @@ amgtar_get_incrname(
 		errmsg = vstrallocf(_("writing to %s: %s"),
 				     incrname, strerror(errno));
 		dbprintf("%s\n", errmsg);
-		return NULL;
+		if (command == CMD_ESTIMATE) {
+		    fprintf(mesgstream, "ERROR %s\n", errmsg);
+		} else {
+		    fprintf(mesgstream, "? %s\n", errmsg);
+		}
+		exit(1);
 	    }
 	}
 
@@ -1442,20 +1457,36 @@ amgtar_get_incrname(
 	    errmsg = vstrallocf(_("reading from %s: %s"),
 			         inputname, strerror(errno));
 	    dbprintf("%s\n", errmsg);
-	    return NULL;
+	    if (command == CMD_ESTIMATE) {
+		fprintf(mesgstream, "ERROR %s\n", errmsg);
+	    } else {
+		fprintf(mesgstream, "? %s\n", errmsg);
+	    }
+	    exit(1);
 	}
 
 	if (close(infd) != 0) {
 	    errmsg = vstrallocf(_("closing %s: %s"),
 			         inputname, strerror(errno));
 	    dbprintf("%s\n", errmsg);
-	    return NULL;
+	    if (command == CMD_ESTIMATE) {
+		fprintf(mesgstream, "ERROR %s\n", errmsg);
+	    } else {
+		fprintf(mesgstream, "? %s\n", errmsg);
+	    }
+	    exit(1);
 	}
 	if (close(outfd) != 0) {
 	    errmsg = vstrallocf(_("closing %s: %s"),
 			         incrname, strerror(errno));
 	    dbprintf("%s\n", errmsg);
-	    return NULL;
+	    dbprintf("%s\n", errmsg);
+	    if (command == CMD_ESTIMATE) {
+		fprintf(mesgstream, "ERROR %s\n", errmsg);
+	    } else {
+		fprintf(mesgstream, "? %s\n", errmsg);
+	    }
+	    exit(1);
 	}
 
 	amfree(inputname);
