@@ -1,5 +1,5 @@
 #!@PERL@
-# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -72,7 +72,6 @@ sub new {
     }
     # default properties
     $self->{'props'} = {
-        'pg-db' => 'template1',
         'pg-cleanupwal' => 'yes',
 	'pg-max-wal-wait' => 60,
     };
@@ -144,6 +143,9 @@ if defined $self->{'args'}->{$pdumpname};
 	$self->{'props'}->{'pg-datadir'} =  $self->{'args'}->{'device'};
     }
 
+    $self->{'args'}->{'verbose'} ||= "no";
+    $self->{'args'}->{'verbose'} = string_to_boolean($self->{'args'}->{'verbose'});
+
     return $self;
 }
 
@@ -223,8 +225,11 @@ sub _run_psql_command {
     push @cmd, "-p", $self->{'props'}->{'pg-port'} if ($self->{'props'}->{'pg-port'});
     push @cmd, "-U", $self->{'props'}->{'pg-user'} if ($self->{'props'}->{'pg-user'});
 
-    push @cmd, '--quiet', '--output', '/dev/null' if (!($cmd =~ /pg_xlogfile_name_offset/));
-    push @cmd, '--command', $cmd, $self->{'props'}->{'pg-db'};
+    if (!($cmd =~ /pg_xlogfile_name_offset/) && !$self->{'args'}->{'verbose'}) {
+	push @cmd, '--quiet', '--output', '/dev/null';
+    }
+    push @cmd, '--command', $cmd;
+    push @cmd,$self->{'props'}->{'pg-db'} if exists $self->{'props'}->{'pg-db'};
     debug("running " . join(" ", @cmd));
 
     my ($wtr, $rdr);
@@ -283,7 +288,6 @@ sub _run_psql_command {
 	}
     });
 
-    close($wtr);
     Amanda::MainLoop::run();
     close($rdr);
     close($err);
@@ -522,7 +526,7 @@ sub _run_tar_totals {
 
     my @cmd;
     @cmd = ($self->{'runtar'}, $self->{'args'}->{'config'},
-        $Amanda::Constants::GNUTAR, '--create', '--totals', @other_args);
+        $Amanda::Constants::GNUTAR, '--create', '--dereference', '--totals', @other_args);
     debug("running: " . join(" ", @cmd));
 
     local (*TAR_IN, *TAR_OUT, *TAR_ERR);
@@ -537,6 +541,10 @@ sub _run_tar_totals {
     while (my $l = <TAR_ERR>) {
         if ($l =~ /^Total bytes written: (\d+)/) {
             $size = $1;
+        } elsif ($l =~ /file changed as we read it/) {
+	    chomp $l;
+	    $self->print_to_server($l, $Amanda::Script_App::GOOD);
+	    debug("TAR_ERR: $l");
         } else {
 	    chomp $l;
 	    $self->print_to_server($l, $Amanda::Script_App::ERROR);
@@ -606,7 +614,7 @@ sub _get_backup_info {
 	       # this works!)
                local *TAROUT;
                my $conf = $self->{'args'}->{'config'} || 'NOCONFIG';
-               my $cmd = "$self->{'runtar'} $conf $Amanda::Constants::GNUTAR --create --file - --directory $self->{'props'}->{'pg-archivedir'} $fname | $Amanda::Constants::GNUTAR --file - --extract --to-stdout";
+               my $cmd = "$self->{'runtar'} $conf $Amanda::Constants::GNUTAR --create --dereference --file - --directory $self->{'props'}->{'pg-archivedir'} $fname | $Amanda::Constants::GNUTAR --file - --extract --to-stdout";
                debug("running: $cmd");
                open(TAROUT, "$cmd |");
                my ($start, $end, $lab);
@@ -659,13 +667,13 @@ sub _get_pg_version {
     my @cmd = ($self->{'props'}->{'psql-path'});
     push @cmd, "-X";
     push @cmd, "--version";
-    my $pid = open3('>&STDIN', \*VERSOUT, '>&STDERR', @cmd)
+    my $pid = open3('>STDIN', \*VERSOUT, '>STDERR', @cmd)
 	or $self->{'die_cb'}->("could not open psql to determine version");
     my @lines = <VERSOUT>;
     waitpid($pid, 0);
     $self->{'die_cb'}->("could not run psql to determine version") if (($? >> 8) != 0);
 
-    my ($maj, $min, $pat) = ($lines[0] =~ / ([0-9]+)\.([0-9]+)\.([0-9]+)$/);
+    my ($maj, $min, $pat) = ($lines[0] =~ / ([0-9]+)\.([0-9]+)\.([0-9]+).*$/);
     return $maj * 10000 + $min * 100 + $pat;
 }
 
@@ -939,6 +947,7 @@ sub command_restore {
 	debug("extracting incremental backup to $cur_dir/$_ARCHIVE_DIR_RESTORE");
 	$status = system($self->{'args'}->{'gnutar-path'},
 		'--extract',
+		'--dereference',
 		'--file', '-',
 		'--ignore-zeros',
 		'--exclude', 'empty-incremental',
@@ -951,6 +960,7 @@ sub command_restore {
 	    mkdir($_DATA_DIR_RESTORE) or die("could not create archive WAL directory: $!");
 	}
 	my @cmd = ($self->{'args'}->{'gnutar-path'}, '--extract',
+		'--dereference',
 		'--file', '-',
 		'--ignore-zero',
 		'--transform', "s,^DATA/,$_DATA_DIR_RESTORE/,S",
@@ -962,6 +972,7 @@ sub command_restore {
 	if (-f $_ARCHIVE_DIR_TAR) {
 	    debug("extracting archive dir to $cur_dir/$_ARCHIVE_DIR_RESTORE");
 	    my @cmd = ($self->{'args'}->{'gnutar-path'}, '--extract',
+		'--dereference',
 		'--exclude', 'empty-incremental',
 		'--file', $_ARCHIVE_DIR_TAR, '--directory',
 		$_ARCHIVE_DIR_RESTORE);
@@ -979,6 +990,7 @@ sub command_restore {
 	if (-f $_DATA_DIR_TAR) {
 	    debug("extracting data dir to $cur_dir/$_DATA_DIR_RESTORE");
 	    my @cmd = ($self->{'args'}->{'gnutar-path'}, '--extract',
+		'--dereference',
 		'--file', $_DATA_DIR_TAR,
 		'--directory', $_DATA_DIR_RESTORE);
 	    debug("run: " . join ' ',@cmd);
@@ -1062,7 +1074,8 @@ GetOptions(
     'passfile=s',
     'port=s',
     'user=s',
-    'psql-path=s'
+    'psql-path=s',
+    'verbose=s'
 ) or usage();
 
 if (defined $opt_version) {
