@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -276,6 +276,15 @@ struct iovec {
 #  include <unistd.h>
 #endif
 
+#if HAVE_EUIDACCESS
+#   define EUIDACCESS euidaccess
+#elif HAVE_EACCESS
+#   define EUIDACCESS eaccess
+#else
+#   define EUIDACCESS access
+#endif
+
+
 #ifdef HAVE_NETINET_IN_H
 #  include <netinet/in.h>
 #endif
@@ -1143,7 +1152,6 @@ extern ssize_t writev(int fd, const struct iovec *iov, int iovcnt);
 #  endif
 #endif
 
-
 #define BIND_CYCLE_RETRIES	120		/* Total of 30 minutes */
 
 #define MAX_DUMPERS 63
