@@ -91,6 +91,9 @@ static int newhandle = 1;
 /*
  * Local functions
  */
+static void bsdtcp_fn_connect(void              *cookie,
+			      security_handle_t *security_handle,
+			      security_status_t  status);
 static int runbsdtcp(struct sec_handle *, in_port_t port);
 
 
@@ -111,6 +114,7 @@ bsdtcp_connect(
     char *canonname;
     char *service;
     in_port_t port;
+    struct addrinfo *res = NULL;
 
     assert(fn != NULL);
     assert(hostname != NULL);
@@ -126,7 +130,7 @@ bsdtcp_connect(
     rh->ev_timeout = NULL;
     rh->rc = NULL;
 
-    result = resolve_hostname(hostname, 0, NULL, &canonname);
+    result = resolve_hostname(hostname, SOCK_STREAM, &res, &canonname);
     if(result != 0) {
 	dbprintf(_("resolve_hostname(%s): %s\n"), hostname, gai_strerror(result));
 	security_seterror(&rh->sech, _("resolve_hostname(%s): %s\n"), hostname,
@@ -172,8 +176,17 @@ bsdtcp_connect(
      *
      * XXX need to eventually limit number of outgoing connections here.
      */
-    if(rh->rc->read == -1) {
-	if (runbsdtcp(rh, port) < 0)
+    rh->res = res;
+    rh->next_res = res;
+    rh->port = port;
+    if (rh->rc->read == -1) {
+	int result = -1;
+	while (rh->next_res) {
+	    result = runbsdtcp(rh, rh->port);
+	    if (result >=0 )
+		break;
+	}
+	if (result < 0)
 	    goto error;
 	rh->rc->refcnt++;
     }
@@ -186,8 +199,10 @@ bsdtcp_connect(
      * Overload rh->rs->ev_read to provide a write event handle.
      * We also register a timeout.
      */
-    rh->fn.connect = fn;
-    rh->arg = arg;
+    rh->fn.connect = &bsdtcp_fn_connect;
+    rh->arg = rh;
+    rh->connect_callback = fn;
+    rh->connect_arg = arg;
     rh->rs->ev_read = event_register((event_id_t)(rh->rs->rc->write),
 	EV_WRITEFD, sec_connect_callback, rh);
     rh->ev_timeout = event_register(CONNECT_TIMEOUT, EV_TIME,
@@ -196,9 +211,65 @@ bsdtcp_connect(
     return;
 
 error:
+    if (res) {
+	freeaddrinfo(res);
+    }
+    rh->res = NULL;
+    rh->next_res = NULL;
     (*fn)(arg, &rh->sech, S_ERROR);
 }
 
+static void
+bsdtcp_fn_connect(
+    void              *cookie,
+    security_handle_t *security_handle,
+    security_status_t  status)
+{
+    struct sec_handle *rh = cookie;
+    int result;
+
+    if (status == S_OK) {
+	int so_errno;
+	socklen_t error_len = sizeof(so_errno);
+	getsockopt(rh->rc->write, SOL_SOCKET, SO_ERROR, &so_errno, &error_len);
+	if ( rh->next_res && so_errno == ECONNREFUSED) {
+	    status = S_ERROR;
+	}
+    }
+
+    switch (status) {
+    case S_TIMEOUT:
+    case S_ERROR:
+	if (rh->next_res) {
+	    while (rh->next_res) {
+		result = runbsdtcp(rh, rh->port);
+		if (result >= 0) {
+		    rh->rc->refcnt++;
+		    rh->rs->ev_read = event_register(
+				(event_id_t)(rh->rs->rc->write),
+				EV_WRITEFD, sec_connect_callback, rh);
+		    rh->ev_timeout = event_register(CONNECT_TIMEOUT, EV_TIME,
+				sec_connect_timeout, rh);
+		    return;
+		}
+	    }
+	}
+	// pass through
+    case S_OK:
+	if (rh->res)
+	    freeaddrinfo(rh->res);
+	rh->res = NULL;
+	rh->next_res = NULL;
+	rh->src_ip = NULL;
+	rh->port = 0;
+	rh->connect_callback(rh->connect_arg, security_handle, status);
+	break;
+    default:
+	assert(0);
+	break;
+    }
+}
+
 /*
  * Setup to handle new incoming connections
  */
@@ -263,18 +334,19 @@ runbsdtcp(
 
     set_root_privs(1);
 
-    server_socket = stream_client_privileged(rc->hostname,
-				     port,
-				     STREAM_BUFSIZE,
-				     STREAM_BUFSIZE,
-				     &my_port,
-				     0);
+    server_socket = stream_client_addr(rh->next_res,
+				       port,
+				       STREAM_BUFSIZE,
+				       STREAM_BUFSIZE,
+				       &my_port,
+				       0,
+				       1);
     set_root_privs(0);
+    rh->next_res = rh->next_res->ai_next;
 
     if(server_socket < 0) {
 	security_seterror(&rh->sech,
 	    "%s", strerror(errno));
-	
 	return -1;
     }
 
