@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-2000 University of Maryland at College Park
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -266,6 +267,7 @@ typedef enum {
     CONFTYPE_PART_CACHE_TYPE,
     CONFTYPE_HOST_LIMIT,
     CONFTYPE_NO_YES_ALL,
+    CONFTYPE_STR_LIST,
 } conftype_t;
 
 typedef enum {
@@ -282,7 +284,7 @@ typedef struct val_s {
         gint64		int64;
         double		r;
         char		*s;
-        ssize_t		size;
+        size_t		size;
         time_t		t;
         float		rate[2];
         exinclude_t	exinclude;
@@ -307,6 +309,7 @@ float                 val_t_to_real     (val_t *);
 char                 *val_t_to_str      (val_t *); /* (also converts CONFTYPE_IDENT) */
 char                 *val_t_to_ident    (val_t *); /* (also converts CONFTYPE_STR) */
 identlist_t           val_t_to_identlist(val_t *);
+identlist_t           val_t_to_str_list (val_t *);
 time_t                val_t_to_time     (val_t *);
 ssize_t               val_t_to_size     (val_t *);
 int                   val_t_to_boolean  (val_t *);
@@ -497,6 +500,9 @@ typedef enum {
     CNF_MAX_DLE_BY_VOLUME,
     CNF_EJECT_VOLUME,
     CNF_TMPDIR,
+    CNF_REPORT_USE_MEDIA,
+    CNF_REPORT_NEXT_MEDIA,
+    CNF_REPORT_FORMAT,
     CNF_CNF /* sentinel */
 } confparm_key;
 
