@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -536,6 +537,9 @@ debug_open(char *subdir)
     char *s = NULL;
     mode_t mask;
 
+    /* create AMANDA_TMPDIR */
+    make_amanda_tmpdir();
+
     /* set up logging while we're here */
     debug_setup_logging();
 
