@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -25,7 +25,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: genversion.c 4946 2012-10-18 11:15:16Z martinea $
+ * $Id: genversion.c 5713 2014-04-25 14:20:01Z martinea $
  *
  * dump the current Amanda version info
  */
@@ -445,6 +445,10 @@ main(
     prvar("UNCOMPRESS_OPT", UNCOMPRESS_OPT);
 #endif
 
+#ifdef CONFIGURE_ARGS
+    prvar("CONFIGURE_ARGS", CONFIGURE_ARGS);
+#endif
+
     endline();
 
     g_printf("  0\n};\n");
