@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 
 #include "amanda.h"
 #include "glib-util.h"
+#include "pthread.h"
 #include "conffile.h" /* For find_multiplier. */
 
 #ifdef HAVE_LIBCURL
@@ -71,7 +72,13 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 static void
 init_ssl(void)
 {
-  gcry_control(GCRYCTL_SET_THREAD_CBS);
+    gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
+
+    if (!gcry_check_version (GCRYPT_VERSION)) {
+	g_critical("libgcrypt version mismatch");
+    }
+
+    gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 }
 
 #else	/* LIBCURL_USE_GNUTLS  */
@@ -100,9 +107,9 @@ glib_init(void) {
      * is initialized) */
 #ifdef HAVE_LIBCURL
 # ifdef G_THREADS_ENABLED
-#  if (GLIB_MAJOR_VERSION < 2 || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 31))
-    g_assert(!g_thread_supported()); /* assert threads aren't initialized yet */
-#  endif
+    if (glib_major_version < 2 ||
+	(glib_major_version == 2 && glib_minor_version < 31))
+	g_assert(!g_thread_supported()); /* assert threads aren't initialized yet */
 # endif
     g_assert(curl_global_init(CURL_GLOBAL_ALL) == 0);
 #endif
