@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 2009 University of Maryland at College Park
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -177,13 +178,13 @@ dvdrw_device_get_type (void)
 
     if G_UNLIKELY(type == 0) {
         static const GTypeInfo info = {
-            sizeof (VfsDeviceClass),
+            sizeof (DvdRwDeviceClass),
             (GBaseInitFunc) NULL,
             (GBaseFinalizeFunc) NULL,
             (GClassInitFunc) dvdrw_device_class_init,
             (GClassFinalizeFunc) NULL,
             NULL /* class_data */,
-            sizeof (VfsDevice),
+            sizeof (DvdRwDevice),
             0 /* n_preallocs */,
             (GInstanceInitFunc) dvdrw_device_init,
             NULL
