@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -142,6 +142,7 @@ gboolean tape_setcompression(int fd, gboolean on);
 gboolean tape_offl(int fd);
 
 DeviceStatusFlags tape_is_tape_device(int fd);
+DeviceStatusFlags get_tape_blocksize(int fd, guint64 *tape_blocksize);
 DeviceStatusFlags tape_is_ready(int fd, TapeDevice *t_self);
 
 #define tape_device_read_size(self) \
@@ -685,6 +686,7 @@ static int try_open_tape_device(TapeDevice * self, char * device_filename) {
     int fd;
     int save_errno;
     DeviceStatusFlags new_status;
+    guint64 tape_blocksize;
 
 #ifdef O_NONBLOCK
     int nonblocking = 0;
@@ -783,6 +785,31 @@ static int try_open_tape_device(TapeDevice * self, char * device_filename) {
         return -1;
     }
 
+    new_status = get_tape_blocksize(fd, &tape_blocksize);
+    if (new_status != DEVICE_STATUS_SUCCESS) {
+	device_set_error(DEVICE(self),
+	    g_strdup_printf(_("Can't get the blocksize of the device %s"),
+			 self->private->device_filename),
+	    new_status);
+        robust_close(fd);
+        return -1;
+    }
+    if (tape_blocksize > 0 && tape_blocksize != tape_device_read_size(self)) {
+	device_set_error(DEVICE(self),
+	    g_strdup_printf(_("Device %s use fixed block size of %lld and tapetype use %lld"), 
+			 self->private->device_filename,
+			 (long long)tape_blocksize,
+			 (long long)tape_device_read_size(self)),
+	    DEVICE_STATUS_VOLUME_ERROR|DEVICE_STATUS_DEVICE_ERROR);
+        robust_close(fd);
+        return -1;
+    }
+    if (tape_blocksize == 0) {
+	g_debug("Device is in variable block size");
+    } else {
+	g_debug("Device is in fixed block size of %lld", (long long)tape_blocksize);
+    }
+
     return fd;
 }
 
@@ -886,7 +913,14 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
     }
 
     buffer_len = tape_device_read_size(self);
-    header_buffer = malloc(buffer_len);
+    header_buffer = g_try_malloc(buffer_len);
+    if (header_buffer == NULL) {
+	device_set_error(dself,
+	    g_strdup(_("Failed to allocate memory")),
+	      DEVICE_STATUS_DEVICE_ERROR
+	    | DEVICE_STATUS_VOLUME_ERROR);
+        return dself->status;
+    }
     result = tape_device_robust_read(self, header_buffer, &buffer_len, &msg);
 
     if (result != RESULT_SUCCESS) {
@@ -927,6 +961,14 @@ static DeviceStatusFlags tape_device_read_label(Device * dself) {
 	return dself->status;
     }
 
+    if (buffer_len < 32768) {
+	device_set_error(dself,
+		g_strdup_printf(_("header is too small: %d bytes"), buffer_len),
+		DEVICE_STATUS_DEVICE_ERROR | DEVICE_STATUS_VOLUME_ERROR);
+	free(header_buffer);
+        return dself->status;
+    }
+
     dself->header_block_size = buffer_len;
     header = dself->volume_header = g_new(dumpfile_t, 1);
     fh_init(header);
@@ -964,7 +1006,13 @@ tape_device_write_block(Device * pself, guint size, gpointer data) {
     /* zero out to the end of a short block -- tape devices only write
      * whole blocks. */
     if (size < pself->block_size) {
-        replacement_buffer = malloc(pself->block_size);
+        replacement_buffer = g_try_malloc(pself->block_size);
+	if (replacement_buffer == NULL) {
+	    device_set_error(pself,
+		g_strdup(_("failed to allocate memory")),
+		DEVICE_STATUS_DEVICE_ERROR);
+	    return FALSE;
+	}
         memcpy(replacement_buffer, data, size);
         bzero(replacement_buffer+size, pself->block_size-size);
 
@@ -1299,6 +1347,14 @@ tape_device_finish_file (Device * d_self) {
     TapeDevice * self;
 
     self = TAPE_DEVICE(d_self);
+
+    if (!d_self->in_file)
+	return TRUE;
+
+    g_mutex_lock(d_self->device_mutex);
+    d_self->in_file = FALSE;
+    g_mutex_unlock(d_self->device_mutex);
+
     if (device_in_error(d_self)) return FALSE;
 
     if (!tape_weof(self->fd, 1)) {
@@ -1310,9 +1366,6 @@ tape_device_finish_file (Device * d_self) {
         return FALSE;
     }
 
-    g_mutex_lock(d_self->device_mutex);
-    d_self->in_file = FALSE;
-    g_mutex_unlock(d_self->device_mutex);
     return TRUE;
 }
 
@@ -1413,7 +1466,14 @@ reseek:
     }
 
     buffer_len = tape_device_read_size(d_self);
-    header_buffer = malloc(buffer_len);
+    header_buffer = g_try_malloc(buffer_len);
+    if (header_buffer == NULL) {
+	device_set_error(d_self,
+		g_strdup(_("failed to allocate memory")),
+		DEVICE_STATUS_DEVICE_ERROR);
+	return NULL;
+    }
+
     d_self->is_eof = FALSE;
     result = tape_device_robust_read(self, header_buffer, &buffer_len, &msg);
 
@@ -1445,6 +1505,14 @@ reseek:
         return NULL;
     }
 
+    if (buffer_len < 32768) {
+	device_set_error(d_self,
+		g_strdup_printf(_("header is too small: %d bytes"), buffer_len),
+		DEVICE_STATUS_DEVICE_ERROR | DEVICE_STATUS_VOLUME_ERROR);
+	free(header_buffer);
+	return NULL;
+    }
+
     rval = g_new(dumpfile_t, 1);
     parse_file_header(header_buffer, rval, buffer_len);
     amfree(header_buffer);
@@ -1777,7 +1845,7 @@ tape_device_robust_write (TapeDevice * self, void * buf, int count, char **errms
 #endif
 	) {
 	    /* if we've retried once already, then we're probably really out of space */
-	    if (retry)
+	    if (retry || !self->leom)
 		return RESULT_NO_SPACE;
 	    retry = TRUE;
 	    d_self->is_eom = TRUE;
@@ -2173,6 +2241,32 @@ DeviceStatusFlags tape_is_tape_device(int fd) {
     }
 }
 
+DeviceStatusFlags get_tape_blocksize(int fd, guint64 *tape_blocksize) {
+   struct mtget status;
+
+   if (ioctl(fd, MTIOCGET, (char *)&status) < 0) {
+	g_debug("get_tape_blocksize: ioctl(MTIOCGET) failed: %s",
+		strerror(errno));
+	*tape_blocksize = -1;
+	return DEVICE_STATUS_DEVICE_ERROR;
+    }
+
+    *tape_blocksize = 0;
+#if defined MT_ST_BLKSIZE_MASK && defined MT_ST_BLKSIZE_SHIFT
+    if (
+#ifdef MT_ISSCSI1
+	status.mt_type == MT_ISSCSI1 ||
+#endif
+#ifdef MT_ISSCSI2
+	status.mt_type == MT_ISSCSI2 ||
+#endif
+	0) {
+	*tape_blocksize = ((status.mt_dsreg & MT_ST_BLKSIZE_MASK) >> MT_ST_BLKSIZE_SHIFT);
+    }
+#endif
+    return DEVICE_STATUS_SUCCESS;
+}
+
 DeviceStatusFlags tape_is_ready(int fd, TapeDevice *t_self G_GNUC_UNUSED) {
     struct mtget get;
     if (0 == ioctl(fd, MTIOCGET, &get)) {
