@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -1024,7 +1024,11 @@ xfer_dest_taper_splitter(
 
     /* set up a ring buffer of size max_memory */
     self->ring_length = max_memory;
-    self->ring_buffer = g_malloc(max_memory);
+    self->ring_buffer = g_try_malloc(max_memory);
+    if (!self->ring_buffer) {
+	g_critical("Can't allocate %llu KB (device-output-buffer-size) of memory", (long long)max_memory/1024);
+    }
+
     self->ring_head = self->ring_tail = 0;
     self->ring_count = 0;
     self->ring_head_at_eof = 0;
