@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -408,6 +408,14 @@ pull_buffer_impl(
 
 	do {
 	    buf = g_malloc(self->block_size);
+	    if (buf == NULL) {
+		xfer_cancel_with_error(elt,
+			_("%s: cannot allocate memory"),
+			self->device->device_name);
+		g_mutex_unlock(self->start_part_mutex);
+		wait_until_xfer_cancelled(elt->xfer);
+		goto error_unlocked;
+	    }
 	    devsize = (int)self->block_size;
 	    result = device_read_block(self->device, buf, &devsize);
 	    *size = devsize;
