@@ -1,8 +1,9 @@
-# Copyright (c) 2009-2012 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda Inc.  All Rights Reserved.
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published
-# by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -562,13 +563,13 @@ sub test_changer {
 	    { slot => 1, state => Amanda::Changer::SLOT_FULL,
 	      barcode => '11111', reserved => 1, loaded_in => 0, current => 1,
 	      device_status => $DEVICE_STATUS_VOLUME_UNLABELED,
-	      device_error => undef,
-	      f_type => undef, label => undef },
+	      device_error => "File 0 not found",
+	      f_type => $Amanda::Header::F_EMPTY, label => undef },
 	    { slot => 2, state => Amanda::Changer::SLOT_FULL,
 	      barcode => '22222', reserved => 1, loaded_in => 1,
 	      device_status => $DEVICE_STATUS_VOLUME_UNLABELED,
-	      device_error => undef,
-	      f_type => undef, label => undef },
+	      device_error => "File 0 not found",
+	      f_type => $Amanda::Header::F_EMPTY, label => undef },
 	    { slot => 3, state => Amanda::Changer::SLOT_FULL,
 	      barcode => '33333',
 	      device_status => undef, device_error => undef,
@@ -1137,7 +1138,7 @@ sub test_changer {
 	      barcode => '33333', loaded_in => 1,
 	      device_status => $DEVICE_STATUS_SUCCESS, 
 	      device_error => undef,
-	      f_type => undef, label => 'TAPE-3' },
+	      f_type => $Amanda::Header::F_TAPESTART, label => 'TAPE-3' },
 	    { slot => 4, state => Amanda::Changer::SLOT_FULL,
 	      barcode => '44444', loaded_in => 0, current => 1,
 	      device_status => $DEVICE_STATUS_SUCCESS,
@@ -1241,7 +1242,7 @@ sub test_changer {
 	      barcode => '33333', loaded_in => 1,
 	      device_status => $DEVICE_STATUS_SUCCESS,
 	      device_error => undef,
-	      f_type => undef, label => 'TAPE-3' },
+	      f_type => $Amanda::Header::F_TAPESTART, label => 'TAPE-3' },
 	    { slot => 4, state => Amanda::Changer::SLOT_FULL,
 	      barcode => '22222', current => 1,
 	      device_status => $DEVICE_STATUS_SUCCESS,
@@ -1308,7 +1309,7 @@ sub test_changer {
 	      barcode => '33333', loaded_in => 0,
 	      device_status => $DEVICE_STATUS_SUCCESS,
 	      device_error => undef,
-	      f_type => undef, label => 'TAPE-3' },
+	      f_type => $Amanda::Header::F_TAPESTART, label => 'TAPE-3' },
 	    { slot => 4, state => Amanda::Changer::SLOT_FULL,
 	      barcode => '22222', current => 1,
 	      device_status => $DEVICE_STATUS_SUCCESS,
@@ -1344,7 +1345,7 @@ sub test_changer {
 		  barcode => '33333', loaded_in => 0,
 		  device_status => $DEVICE_STATUS_SUCCESS,
 		  device_error => undef,
-		  f_type => undef, label => 'TAPE-3' },
+		  f_type => $Amanda::Header::F_TAPESTART, label => 'TAPE-3' },
 		{ slot => 4, state => Amanda::Changer::SLOT_FULL,
 		  barcode => '22222', current => 1,
 		  device_status => $DEVICE_STATUS_SUCCESS,
@@ -1355,7 +1356,7 @@ sub test_changer {
 		  device_status => $DEVICE_STATUS_SUCCESS,
 		  device_error => undef,
 		  f_type => $Amanda::Header::F_TAPESTART, label => 'TAPE-4' },
-	    ], "$pfx: inventory reflects updates with unknown state with barcodes");
+	    ], "$pfx: inventory reflects updates wrcodesith unknown state with barcodes");
 	} else {
 	    check_inventory($chg, $mtx_config->{'barcodes'} > 0, $steps->{'quit'}, [
 		{ slot => 1, state => Amanda::Changer::SLOT_FULL,
