@@ -1,4 +1,4 @@
-# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -89,7 +89,7 @@ die "$chg" if $chg->isa("Amanda::Changer::Error");
 	chg_err_like($err,
 	    { message => qr{'null:/foo' is already reserved},
 	      type => 'failed',
-	      reason => 'volinuse' },
+	      reason => 'driveinuse' },
 	    "second simultaneous reservation rejected");
 
 	# release the first reservation
