@@ -1,8 +1,9 @@
 # Copyright (c) Zmanda Inc.  All Rights Reserved.
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published
-# by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -66,6 +67,7 @@ set_tapelist(<<EOF);
 20090424173002 TEST-2 reuse
 20090424173003 TEST-3 reuse
 20090424173004 TEST-4 reuse
+20090424172000 CONF-4 reuse
 EOF
 $taperscan->read_tapelist();
 
