@@ -1,4 +1,4 @@
-# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -268,9 +268,9 @@ pass("Two simultaneous transfers run to completion");
     my $xfer = Amanda::Xfer->new([
 	Amanda::Xfer::Source::Random->new(1024*1024, $RANDOM_SEED),
 	Amanda::Xfer::Filter::Process->new(
-	    [ $Amanda::Constants::COMPRESS_PATH, $Amanda::Constants::COMPRESS_BEST_OPT ], 0),
+	    [ $Amanda::Constants::COMPRESS_PATH, $Amanda::Constants::COMPRESS_BEST_OPT ], 0, 0, 0, 0),
 	Amanda::Xfer::Filter::Process->new(
-	    [ $Amanda::Constants::UNCOMPRESS_PATH, $Amanda::Constants::UNCOMPRESS_OPT ], 0),
+	    [ $Amanda::Constants::UNCOMPRESS_PATH, $Amanda::Constants::UNCOMPRESS_OPT ], 0, 0, 0, 0),
 	Amanda::Xfer::Dest::Null->new($RANDOM_SEED),
     ]);
 
@@ -300,7 +300,7 @@ pass("Two simultaneous transfers run to completion");
     my $xfer = Amanda::Xfer->new([
 	Amanda::Xfer::Source::Fd->new($zerofd),
 	Amanda::Xfer::Filter::Process->new(
-	    [ $Amanda::Constants::COMPRESS_PATH, $Amanda::Constants::COMPRESS_BEST_OPT ], 0),
+	    [ $Amanda::Constants::COMPRESS_PATH, $Amanda::Constants::COMPRESS_BEST_OPT ], 0, 0, 0, 0),
 	Amanda::Xfer::Dest::Null->new(0),
     ]);
 
@@ -1099,6 +1099,7 @@ SKIP: {
     my $mkdevice = sub {
 	my $dev = Amanda::Device->new("ndmp:127.0.0.1:$ndmp_port\@$drive");
 	die "can't create device" unless $dev->status() == $Amanda::Device::DEVICE_STATUS_SUCCESS;
+	$dev->property_set("indirect", 0) or die "can't set INDIRECT";
 	$dev->property_set("verbose", 1) or die "can't set VERBOSE";
 	$dev->property_set("ndmp_username", "ndmp") or die "can't set username";
 	$dev->property_set("ndmp_password", "ndmp") or die "can't set password";
