@@ -150,10 +150,11 @@ process_args (int argc, char *argv[])
 	char **		pp;
 	char *		p;
 	char *		op;
-	char *		av[1000];
+	char **		av;
 	int		ac = 0;
 
 	progname = argv[0];
+	av = malloc((argc+1000) * sizeof(char *));
 
 	if (argc == 2 && strcmp (argv[1], "-help") == 0) {
 		help();
@@ -674,6 +675,7 @@ handle_long_option (char *str)
 		d_agent[size] = '\0';
 		if (size > 0 && d_agent[size-1] == '\n')
 		    d_agent[size-1] = '\0';
+		close(fd);
 		if (ndmagent_from_str (&D_data_agent, d_agent)) {
 			error_byebye ("bad -D-agent-fd argument");
 		}
