@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -485,7 +485,7 @@ ndmp_connection_mover_listen(
 		ndmp4_tcp_addr *na = &reply->connect_addr.ndmp4_addr_u.tcp_addr.tcp_addr_val[i];
 		(*addrs)[i].sin.sin_family = AF_INET;
 		(*addrs)[i].sin.sin_addr.s_addr = htonl(na->ip_addr);
-		SU_SET_PORT(addrs[i], na->port);
+		SU_SET_PORT(&((*addrs)[i]), na->port);
 	    }
 	}
 	NDMP_FREE();
@@ -847,7 +847,7 @@ ndmp_connection_wait_for_notify_with_cond(
     }
 
     if (found)
-	return TRUE;
+	return 1;
 
 	/* otherwise, wait for an incoming packet and handle it, then try
 	 * again.  There's some select trickery here to avoid hogging the
