@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -89,6 +89,7 @@ sub get_device {
     my ($device_name) = @_;
 
     my $device = Amanda::Changer::robot::get_device($self, $device_name);
+    return $device if $device->isa("Amanda::Changer::Error");
 
     # set the authentication properties for the new device based on our
     # own settings, but only if they haven't been set by the user
@@ -314,7 +315,7 @@ sub _do_move_medium {
 	}
 
 	unless (defined $dst_elem) {
-	    return $finished_cb->("unknown destiation slot/drive '$dst'");
+	    return $finished_cb->("unknown destination slot/drive '$dst'");
 	}
 
 	# send a MOVE MEDIUM command
