@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -413,6 +413,7 @@ sub load_unlocked {
         if (defined $slot and !exists $state->{'slots'}->{$slot}) {
             return $self->make_error("failed", $params{'res_cb'},
                     reason => "invalid",
+		    slot   => $slot,
                     message => "invalid slot '$slot'");
         }
 
@@ -438,11 +439,13 @@ sub load_unlocked {
 	    if (exists $params{'label'}) {
 		return $self->make_error("failed", $params{'res_cb'},
 			reason => "invalid",
+			slot   => $slot,
 			message => "label '$params{label}' is in slot $slot, which is " .
 				   "not in use-slots ($self->{use_slots})");
 	    } else {
 		return $self->make_error("failed", $params{'res_cb'},
 			reason => "invalid",
+			slot   => $slot,
 			message => "slot $slot not in use-slots ($self->{use_slots})");
 	    }
 	}
@@ -1067,6 +1070,12 @@ sub eject_unlocked {
 		    message => "tape in drive '$drive' is in use");
 	}
 
+	if (!defined $drive_info->{'orig_slot'}) {
+	    return $self->make_error("failed", $params{'finished_cb'},
+			reason => "invalid",
+			message => "drive '$drive' is empty");
+	}
+
 	if ($self->{'eject_before_unload'}) {
 	    $steps->{'wait_to_eject'}->();
 	} else {
@@ -1460,6 +1469,226 @@ sub move_unlocked {
     }
 }
 
+sub verify {
+    my $self = shift;
+    my %params = @_;
+
+    return if $self->check_error($params{'finished_cb'});
+
+    $self->_with_updated_state(\%params, 'finished_cb',
+	sub { $self->verify_unlocked(@_); });
+}
+
+sub verify_unlocked {
+    my $self = shift;
+    my %params = @_;
+    my $state = $params{'state'};
+    my @drives;
+    my @devices;
+    my $slot;
+    my $drive;
+    my @results;
+    my @tape_devices;
+
+    my $steps = define_steps
+	cb_ref => \$params{'finished_cb'};
+
+    step unload_all => sub {
+	# get a drive to eject
+	for my $drive (keys %{$state->{'drives'}}) {
+	    if ($state->{'drives'}->{$drive}->{'state'} == Amanda::Changer::SLOT_FULL) {
+		return $self->eject_unlocked(drive => $drive,
+			      state => $state,
+			      finished_cb => $steps->{'unload_all_unloaded'});
+	    }
+	}
+
+	# get a new state
+	$params{'finished_get_state_cb'} = $steps->{'verify_all_unloaded'};
+	return $self->_get_state(\%params);
+    };
+
+    step unload_all_unloaded => sub {
+	my ($err) = @_;
+
+	if ($err) {
+	    return $params{'finished_cb'}->($err);
+	}
+	return $steps->{'unload_all'}->();
+    };
+
+    step verify_all_unloaded => sub {
+	my ($newstate) = @_;
+
+	if (UNIVERSAL::isa($state, 'Amanda::Changer::Error')) {
+	    return $params{'finished_cb'}->($newstate);
+	}
+
+	my @loaded_drive;
+
+	# verify all drives are unloaded
+	for my $drive (keys %{$state->{'drives'}}) {
+	    push @drives, $drive;
+	    if ($state->{'drives'}->{$drive}->{'state'} == Amanda::Changer::SLOT_FULL) {
+		push @loaded_drive, $drive;
+	    }
+	    if ($self->{'drive2device'}->{$drive}) {
+		push @devices, $self->{'drive2device'}->{$drive};
+	    }
+	}
+	if (@loaded_drive) {
+	    return $self->make_error("failed", $params{'finished_cb'},
+			reason => "invalid",
+			message => "Can't unload all drives: " .
+				   join(" ", @loaded_drive));
+	}
+	$steps->{'eject_devices'}->();
+    };
+
+    step eject_devices => sub {
+	return $steps->{'verify_a_drive'}->() if !@devices;
+
+	my $device_name = pop @devices;
+	my $device = $self->get_device($device_name);
+	if ($device->isa("Amanda::Changer::Error")) {
+	    push @results, "$device";
+	    return $steps->{'eject_devices'}->();
+	}
+	$device->eject();
+	return $steps->{'eject_devices'}->();
+    };
+
+    step verify_a_drive => sub {
+	return $steps->{'all_done'}->() if !@drives;
+
+	$drive = pop @drives;
+	if (!defined $self->{'drive2device'}->{$drive}) {
+	    return $steps->{'verify_a_drive'}->();
+	}
+
+	# find a full slot in the allowed range
+	$slot = $self->_get_next_slot($state, -1);
+
+	$self->{'interface'}->load($slot, $drive, $steps->{'load_finished'});
+    };
+
+    step load_finished => sub {
+	my ($err) = @_;
+	if ($err) {
+	    push @results, "$err";
+	    return $steps->{'verify_a_drive'}->();
+	}
+	$steps->{'start_polling'}->();
+    };
+
+    my ($next_poll, $last_poll);
+    step start_polling => sub {
+	my ($delay, $poll, $until) = @{ $self->{'load_poll'} };
+	my $now = time;
+	$next_poll = $now + $delay;
+	$last_poll = $now + $until;
+
+	return Amanda::MainLoop::call_after(1000 * ($next_poll - $now), $steps->{'check_device'});
+    };
+
+    step check_device => sub {
+	my $device_name = $self->{'drive2device'}->{$drive};
+	confess "drive $drive not found in drive2device" unless $device_name; # shouldn't happen
+
+	my ($delay, $poll, $until) = @{ $self->{'load_poll'} };
+	$self->_debug("polling '$device_name' to see if it's ready");
+
+	my $device = $self->get_device($device_name);
+	if ($device->isa("Amanda::Changer::Error")) {
+	    push @results, "$device";
+	    return $steps->{'unload'}->();
+	}
+
+	$device->read_label();
+
+	my $device_name2;
+	my $hint = 0;
+	# see if the device thinks it's possible it's busy or empty
+	if ($device->status & $DEVICE_STATUS_VOLUME_MISSING
+	    or $device->status & $DEVICE_STATUS_DEVICE_BUSY) {
+
+	    # check if any other device is loaded, to abort the poll sooner
+	    for my $ddrive (keys %{$state->{'drives'}}) {
+		#next if $ddrive == $drive;
+		if (defined $self->{'drive2device'}->{$ddrive}) {
+		    $device_name2 = $self->{'drive2device'}->{$ddrive};
+		    my $device2 = $self->get_device($device_name2);
+		    next if $device2->isa("Amanda::Changer::Error");
+
+		    $device2->read_label();
+
+		    if (!($device2->status & $DEVICE_STATUS_VOLUME_MISSING) &&
+			!($device2->status & $DEVICE_STATUS_DEVICE_ERROR)) {
+			$hint = 1;
+			$last_poll -= $until;
+			last;
+		    }
+		}
+	    }
+
+	    # device is not ready -- set up for the next polling step
+	    my $now = time;
+	    $next_poll += $poll;
+	    $next_poll = $now + 1 if ($next_poll < $now);
+	    if ($poll != 0 and $next_poll < $last_poll) {
+		return Amanda::MainLoop::call_after(
+			1000 * ($next_poll - $now), $steps->{'check_device'});
+	    }
+
+	    # (fall through if we're done polling)
+	}
+
+	if ($device->status & $DEVICE_STATUS_VOLUME_MISSING) {
+	    debug("ERROR: Drive $drive is not device $device_name");
+	    push @results, "ERROR: Drive $drive is not device $device_name";
+	} elsif ($device->status & $DEVICE_STATUS_DEVICE_ERROR) {
+	    debug("ERROR: Drive $drive: " . $device->error());
+	    push @results, "ERROR: Drive $drive: " . $device->error();
+	} else {
+	    debug("GOOD : Drive $drive is device $device_name");
+	    push @results, "GOOD : Drive $drive is device $device_name";
+	    push @tape_devices, "$drive=$device_name";
+	}
+	if ($hint) {
+	    debug("HINT : Drive $drive looks to be device $device_name2");
+	    push @results, "HINT : Drive $drive looks to be device $device_name2";
+	    push @tape_devices, "$drive=$device_name2";
+	}
+
+	return $steps->{'unload'}->();
+    };
+
+    step unload => sub {
+	$self->{'interface'}->unload($drive, $slot, $steps->{'unload_done'});
+    };
+
+    step unload_done => sub {
+	return $steps->{'verify_a_drive'}->();
+    };
+
+    step all_done => sub {
+	my $tape_devices;
+
+	for my $drive (@{$self->{'driveorder'}}) {
+	    if (!exists $state->{'drives'}->{$drive}) {
+		debug("ERROR: Drive $drive: no such drive in changer");
+		push @results, "ERROR: Drive $drive: no such drive in changer";
+	    }
+	}
+
+	foreach my $tape_device (@tape_devices) {
+	    $tape_devices .= " \"$tape_device\"";
+	}
+	push @results, "property \"TAPE-DEVICE\"$tape_devices";
+	$params{'finished_cb'}->(undef, @results);
+    };
+}
+
 ##
 # Utilities
 
@@ -1567,15 +1796,45 @@ sub _with_updated_state {
 
     step start => sub {
 	$self->with_locked_state($self->{'statefile'},
-	    $params{$cbname}, $steps->{'got_lock'});
+		    $params{$cbname}, $steps->{'got_lock'});
     };
 
     step got_lock => sub {
 	($state, my $new_cb) = @_;
 
 	# set up params for calling through to $sub later
-	$params{'state'} = $state;
 	$params{$cbname} = $new_cb;
+	$paramsref->{$cbname} = $new_cb;
+	$paramsref->{'state'} = $state;
+	$paramsref->{'finished_get_state_cb'} = $steps->{'got_state'};
+	$self->_get_state($paramsref);
+    };
+
+    step got_state => sub {
+	my ($state) = @_;
+
+	if (UNIVERSAL::isa($state, 'Amanda::Changer::Error')) {
+	    return $paramsref->{$cbname}->($state);
+	}
+
+	$params{'state'} = $state;
+	# finally, call through to the user's method; $params{$cbname} has been
+	# properly patched to release the state lock when this method is done.
+	$sub->(%params);
+    }
+}
+
+sub _get_state {
+    my $self = shift;
+    my ($paramsref) = @_;
+    my %params = %$paramsref;
+
+    my $state = $params{'state'};
+
+    my $steps = define_steps
+        cb_ref => \$params{'finished_get_state_cb'};
+
+    step got_lock => sub {
 
 	if (!keys %$state) {
 	    $state->{'slots'} = {};
@@ -1597,7 +1856,8 @@ sub _with_updated_state {
 	# if it's not time for another run of the status command yet, then just skip to
 	# the end.
 	if (defined $state->{'last_status'}
-	    and time < $state->{'last_status'} + $self->{'status_interval'}) {
+	    and time < $state->{'last_status'} + $self->{'status_interval'}
+	    and  defined $self->{'got_status'}) {
 	    $self->_debug("too early for another 'status' invocation");
 	    $steps->{'done'}->();
 	} else {
@@ -1616,7 +1876,8 @@ sub _with_updated_state {
     step status_cb => sub {
 	my ($err, $status) = @_;
 	if ($err) {
-	    return $self->make_error("fatal", $params{$cbname},
+	    return $self->make_error("failed", $params{'finished_get_state_cb'},
+		reason => "invalid",
 		message => $err);
 	}
 
@@ -1650,7 +1911,22 @@ sub _with_updated_state {
 	    if (defined $info->{'barcode'}) {
 
                 my $label = $state->{'bc2lb'}->{$info->{'barcode'}};
+		my $tl_label;
+		if (defined $self->{'tapelist'}) {
+		    my $tle = $self->{'tapelist'}->lookup_by_barcode($info->{'barcode'});
+		    if (defined $tle) {
+			$tl_label = $tle->{'label'};
+		    }
+		}
 
+		if (defined $label and defined $tl_label and
+		    $label ne $tl_label) {
+		    debug("MISMATCH label for barcode  state ($label)   tapelist ($tl_label) for barcode $info->{'barcode'}");
+		}
+		if (!defined $label && defined $tl_label) {
+		    $label = $tl_label;
+		    $state->{'bc2lb'}->{$info->{'barcode'}} = $tl_label;
+		}
 		$new_slots->{$slot} = {
                     state => Amanda::Changer::SLOT_FULL,
 		    device_status => $state->{'slots'}->{$slot}->{device_status},
@@ -1664,7 +1940,9 @@ sub _with_updated_state {
 	    } else {
 		# assume the status of this slot has not changed since the last
                 # time we looked at it, although mark it as not loaded in a slot
-		if (exists $state->{'slots'}->{$slot}) {
+		if (exists $state->{'slots'}->{$slot} and
+		    exists $state->{'slots'}->{$slot}->{'state'} and
+		    $state->{'slots'}->{$slot}->{'state'} == Amanda::Changer::SLOT_FULL) {
 		    $new_slots->{$slot} = $state->{'slots'}->{$slot};
 		    $new_slots->{$slot}->{'loaded_in'} = undef;
 		} else {
@@ -1748,6 +2026,21 @@ sub _with_updated_state {
 	    # and look up the label by barcode if possible
 	    if (!defined $label && defined $info->{'barcode'}) {
 		$label = $state->{'bc2lb'}->{$info->{'barcode'}};
+		my $tl_label;
+		if (defined $self->{'tapelist'}) {
+		    my $tle = $self->{'tapelist'}->lookup_by_barcode($info->{'barcode'});
+		    if (defined $tle) {
+			$tl_label = $tle->{'label'};
+		    }
+		}
+
+		if (defined $label and defined $tl_label) {
+		    debug("MISMATCH drive label for barcode  state ($label)   tapelist ($tl_label) for barcode $info->{'barcode'}");
+		}
+		if (!defined $label && defined $tl_label) {
+		    $label = $tl_label;
+		    $state->{'bc2lb'}->{$info->{'barcode'}} = $tl_label;
+		}
 	    }
 
 	    $new_drives->{$drv} = {
@@ -1793,9 +2086,8 @@ sub _with_updated_state {
     };
 
     step done => sub {
-	# finally, call through to the user's method; $params{$cbname} has been
-	# properly patched to release the state lock when this method is done.
-	$sub->(%params);
+	$self->{'got_status'} = 1;
+	$params{'finished_get_state_cb'}->($state);
     };
 }
 
@@ -2006,12 +2298,15 @@ sub status {
 	    my ($exitstatus, $output) = @_;
 	    if ($exitstatus != 0) {
 		my $err = $output;
+		for my $line (split '\n', $output) {
+		    debug("mtx: $line");
+		}
 		# if it's a regular SCSI error, just show the sense key
 		my ($sensekey) = ($err =~ /mtx: Request Sense: Sense Key=(.*)\n/);
 		$err = "SCSI error; Sense Key=$sensekey" if $sensekey;
 		$counter--;
-		if ($sensekey eq "Not Ready" and $counter > 0) {
-		    debug("$output");
+		if (($sensekey eq "Not Ready" and $counter > 0) ||
+		    ($sensekey eq "No Sense" and $counter > 0)) {
 		    return Amanda::MainLoop::call_after(1000, $run_mtx);
 		}
 		return $status_cb->("error from mtx: " . $err, {});
