@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -264,6 +264,10 @@ sub find_volume {
     };
 
     step get_inventory => sub {
+	if ($remove_undef_state and $self->{'chg'}->{'scan-require-update'}) {
+	    Amanda::Debug::debug("update the changer");
+	    $self->{'chg'}->update();
+	}
 	$self->{'chg'}->inventory(inventory_cb => $steps->{'parse_inventory'});
     };
 
@@ -407,6 +411,9 @@ sub find_volume {
 	if ($last_err) {
 	    return $steps->{'handle_error'}->($last_err, undef);
 	} else {
+Amanda::Debug::debug("notfound 1");
+Amanda::Debug::debug("label $label");
+Amanda::Debug::debug("inventory: " . Data::Dumper::Dumper($inventory));
 	    return $steps->{'handle_error'}->(
 		    Amanda::Changer::Error->new('failed',
 			    reason => 'notfound',
