@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -571,7 +571,7 @@ sub get_hosts
     my $self  = shift @_;
     my $cache = $self->{cache};
 
-    $cache->{hosts} = [ keys %{ $self->{data}{disklist} } ]
+    $cache->{hosts} = [ sort keys %{ $self->{data}{disklist} } ]
       if ( !defined $cache->{hosts} );
 
     return @{ $cache->{hosts} };
@@ -581,7 +581,7 @@ sub get_disks
 {
     my $self = shift @_;
     my ($hostname) = @_;
-    return keys %{ $self->{data}{disklist}{$hostname} };
+    return sort keys %{ $self->{data}{disklist}{$hostname} };
 }
 
 sub get_dles
@@ -599,13 +599,6 @@ sub get_dles
     return @{ $cache->{dles} };
 }
 
-sub xml_output
-{
-    my ( $self, $org, $config ) = @_;
-    use Amanda::Report::xml;
-    return Amanda::Report::xml::make_amreport_xml( $self, $org, $config );
-}
-
 sub get_dle_info
 {
     my $self = shift @_;
