@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -126,6 +126,7 @@ sub _scan {
     my $scan_running = 0;
     my $interactivity_running = 0;
     my $restart_scan = 0;
+    my $restart_scan_changer = undef;
     my $abort_scan = undef;
     my $last_err = undef; # keep the last meaningful error, the one reported
 			  # to the user, most scan end with the notfound error,
@@ -160,10 +161,22 @@ sub _scan {
 
     step restart_scan => sub {
 	$restart_scan = 0;
+
+	# Reload the tapelist at every scan.
+	$self->{'tapelist'}->reload(0);
+
+	if ($restart_scan_changer) {
+	    $self->{'chg'}->quit() if $self->{'chg'} != $self->{'initial_chg'};
+	    $self->{'chg'} = $restart_scan_changer;
+	    $restart_scan_changer = undef;
+	}
 	return $steps->{'get_inventory'}->();
     };
 
     step get_inventory => sub {
+	if ($remove_undef_state and $self->{'chg'}->{'scan-require-update'}) {
+	    $self->{'chg'}->update();
+	}
 	$self->{'chg'}->inventory(inventory_cb => $steps->{'parse_inventory'});
     };
 
@@ -173,6 +186,9 @@ sub _scan {
 	if ($err && $err->notimpl) {
 	    #inventory not implemented
 	    die("no inventory");
+	} elsif ($err and $err->fatal) {
+	    #inventory fail
+	    return $steps->{'call_result_cb'}->($err, undef);
 	}
 	return $steps->{'handle_error'}->($err, undef) if $err;
 
@@ -222,6 +238,7 @@ sub _scan {
 
 	if (defined $res) {
 	    $res->release(finished_cb => $steps->{'released'});
+	    $res = undef;
 	} else {
 	    $steps->{'released'}->();
 	}
@@ -232,7 +249,7 @@ sub _scan {
 	    $slot_scanned = $action_slot;
 	    $self->_user_msg(scan_slot => 1,
 			     slot => $slot_scanned);
-	    return $self->{'changer'}->load(
+	    return $self->{'chg'}->load(
 			slot => $slot_scanned,
 			set_current => $params{'set_current'},
 			res_cb => $steps->{'slot_loaded'});
@@ -405,9 +422,11 @@ sub _scan {
     };
 
     step after_poll => sub {
-	$poll_src->remove() if defined $poll_src;
-	$poll_src = undef;
-	return $steps->{'restart_scan'}->();
+	if ($poll_src) {
+	    $poll_src->remove();
+	    $poll_src = undef;
+	    return $steps->{'restart_scan'}->();
+	}
     };
 
     step scan_interactivity => sub {
@@ -448,6 +467,9 @@ sub _scan {
 	    }
 	}
 
+	# remove leading and trailing space
+	$message =~ s/^ +//g;
+	$message =~ s/ +$//g;
 	if ($message ne '') {
 	    # use a new changer
 	    my $new_chg;
@@ -459,8 +481,7 @@ sub _scan {
 	    if ($new_chg->isa("Amanda::Changer::Error")) {
 		return $steps->{'scan_interactivity'}->("$new_chg");
 	    }
-	    $self->{'chg'}->quit() if $self->{'chg'} != $self->{'initial_chg'};
-	    $self->{'chg'} = $new_chg;
+	    $restart_scan_changer = $new_chg;
 	    %seen = ();
 	} else {
 	    $remove_undef_state = 1;
@@ -497,7 +518,8 @@ sub _scan {
 	$poll_src = undef;
 	$interactivity_running = 0;
 	$self->{'interactivity'}->abort() if defined $self->{'interactivity'};
-	$self->{'chg'}->quit() if $self->{'chg'} != $self->{'initial_chg'} and !$res;
+	$self->{'chg'}->quit() if $self->{'chg'} != $self->{'initial_chg'} and
+				  !$res;
 	if ($err) {
 	    $self->{'scanning'} = 0;
 	    return $result_cb->($err, $res);
@@ -561,6 +583,7 @@ sub volume_is_labelable {
 	if (!$autolabel->{'volume_error'}) {
 	    $self->_user_msg(slot_result  => 1,
 			     volume_error => 1,
+			     err          => $sl->{'device_error'},
 			     slot         => $slot);
 	    return 0;
 	}
@@ -571,15 +594,25 @@ sub volume_is_labelable {
 			     slot         => $slot);
 	return 0;
     } elsif ($dev_status == $DEVICE_STATUS_SUCCESS and
-	     $f_type == $Amanda::Header::F_TAPESTART and
-	     $label !~ /$self->{'labelstr'}/) {
-	if (!$autolabel->{'other_config'}) {
-	    $self->_user_msg(slot_result  => 1,
-			     label        => $label,
-			     labelstr     => $self->{'labelstr'},
-			     does_not_match_labelstr => 1,
-			     slot         => $slot);
-	    return 0;
+	     $f_type == $Amanda::Header::F_TAPESTART) {
+	if ($label !~ /$self->{'labelstr'}/) {
+	    if (!$autolabel->{'other_config'}) {
+		$self->_user_msg(slot_result  => 1,
+				 label        => $label,
+				 labelstr     => $self->{'labelstr'},
+				 does_not_match_labelstr => 1,
+				 slot         => $slot);
+		return 0;
+	    }
+	} else {
+	   my $vol_tle = $self->{'tapelist'}->lookup_tapelabel($label);
+	   if (!$vol_tle) {
+		$self->_user_msg(slot_result  => 1,
+				 label        => $label,
+				 not_in_tapelist => 1,
+				 slot         => $slot);
+		return 0;
+	   }
 	}
     }
 
