@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -316,6 +316,13 @@ sub quit {
     }
 }
 
+sub set_write_timestamp {
+    my $self = shift;
+    my $write_timestamp = shift;
+
+    $self->{'write_timestamp'} = $write_timestamp;
+}
+
 sub scan {
     my $self = shift;
     my %params = @_;
@@ -361,6 +368,11 @@ sub is_reusable_volume {
 	return $params{'new_label_ok'};
     }
 
+    if (defined $self->{'write_timestamp'} and
+	$vol_tle->{'datestamp'} eq $self->{'write_timestamp'}) {
+	return 0;
+    }
+
     # see if it's in the collection of reusable volumes
     my @tapelist = @{$self->{'tapelist'}->{'tles'}};
     my @reusable = @tapelist[$self->{'tapecycle'}-1 .. $#tapelist];
