@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -503,6 +503,7 @@ sub start {
 
     $self->dbg("starting");
     $self->{'write_timestamp'} = $params{'write_timestamp'};
+    $self->{'taperscan'}->set_write_timestamp($self->{'write_timestamp'});
 
     # start up the DevHandling object, making sure we know
     # when it's done with its startup process
@@ -512,6 +513,35 @@ sub start {
     });
 }
 
+sub wait_device {
+    my $self = shift;
+    my %params = @_;
+
+    for my $rq_param (qw(finished_cb)) {
+	croak "required parameter '$rq_param' missing"
+	    unless exists $params{$rq_param};
+    }
+
+    $self->{'cancelled'} = 0;
+
+    if ($self->get_device()) {
+	return $params{'finished_cb'}->(undef);
+    }
+
+    if ($self->{'devhandling'}->{'scan_running'}) {
+	$self->{'devhandling'}->{'start_finished_cb'} = $params{'finished_cb'};
+	return;
+    }
+
+    # start up the DevHandling object, making sure we know
+    # when it's done with its startup process
+    $self->{'started'} = 0;
+    $self->{'devhandling'}->start(finished_cb => sub {
+	$self->{'started'} = 1;
+	$params{'finished_cb'}->(@_);
+    });
+}
+
 sub quit {
     my $self = shift;
     my %params = @_;
@@ -591,7 +621,7 @@ sub check_data_path {
     my $device = $self->get_device();
 
     if (!defined $device) {
-	confess "no device is available to check the datapath";
+	return "no device is available to check the datapath";
     }
 
     my $use_directtcp = $device->directtcp_supported();
@@ -634,7 +664,7 @@ sub get_xfer_dest {
     $self->{'xdt'} = undef;
     $self->{'size'} = 0;
     $self->{'duration'} = 0.0;
-    $self->{'nparts'} = undef;
+    $self->{'nparts'} = 0;
     $self->{'dump_start_time'} = undef;
     $self->{'last_part_successful'} = 1;
     $self->{'started_writing'} = 0;
@@ -771,6 +801,7 @@ sub cancel_dump {
     confess "no xfer dest set up; call get_xfer_dest first"
 	unless defined $self->{'xdt'};
 
+    $self->{'devhandling'}->{'volume_cb'} = undef;
     # set up the dump_cb for when this dump is done, and keep the xfer
     $self->{'dump_cb'} = $params{'dump_cb'};
     $self->{'xfer'} = $params{'xfer'};
@@ -778,6 +809,7 @@ sub cancel_dump {
     # The cancel will can dump_cb.
 
     $self->{'xfer'}->cancel();
+    $self->{'cancelled'} = 1;
 
 }
 
@@ -836,6 +868,9 @@ sub _start_part {
     # and start writing this part
     $self->{'started_writing'} = 1;
     $self->dbg("resuming transfer");
+    if ($self->{'nparts'} == 0) {
+	$self->{'feedback'}->scribe_ready();
+    }
     $self->{'xdt'}->start_part(!$self->{'last_part_successful'},
 			       $self->{'dump_header'});
 }
@@ -871,7 +906,7 @@ sub _xmsg_part_done {
     # are an implementation detail of the splitting done by the transfer
     # destination.
 
-    if ($msg->{'successful'} and $msg->{'size'} == 0) {
+    if ($msg->{'successful'} and $msg->{'size'} == 0 and $msg->{'partnum'} == 0) {
 	$self->dbg("not notifying for empty, successful part");
     } else {
 	# double-check partnum
@@ -1147,6 +1182,7 @@ sub _get_new_volume {
 	return;
     }
 
+    return if $self->{'cancelled'};
     $self->{'devhandling'}->get_volume(volume_cb => sub { $self->_volume_cb(@_); });
 }
 
@@ -1391,7 +1427,8 @@ sub _device_start {
 	    my $tle = $tl->lookup_tapelabel($new_label);
 	    $meta = $tle->{'meta'} if !defined $meta && $tle->{'meta'};
 	    my $barcode = $tle->{'barcode'};
-	    if (defined $barcode and $barcode ne $reservation->{'barcode'}) {
+	    if (defined $barcode and defined $reservation->{'barcode'} and
+		$barcode ne $reservation->{'barcode'}) {
 		return $finished_cb->("tapelist for label '$new_label' have barcode '$barcode' but changer report '" . $reservation->{'barcode'} . "'");
 	    }
 	}
@@ -1929,6 +1966,8 @@ sub _maybe_callback {
 	$self->{'new_scribe'} = undef;
 
 	$volume_cb->(@volume_cb_args);
+    } elsif (!$self->{'volume_cb'}) {
+	$self->{'scan_finished'} = 0;
     }
 }
 
