@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2012 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -439,6 +439,14 @@ sub scribe_notif_new_tape {
     }
 }
 
+sub scribe_ready {
+    my $self = shift;
+    my %params = @_;
+
+    $self->{'controller'}->{'proto'}->send(Amanda::Taper::Protocol::READY,
+	handle => $self->{'handle'});
+}
+
 sub scribe_notif_part_done {
     my $self = shift;
     my %params = @_;
@@ -597,6 +605,7 @@ sub send_port_and_get_header {
 sub setup_and_start_dump {
     my $self = shift;
     my ($msgtype, %params) = @_;
+    my %splitting_args;
     my %get_xfer_dest_args;
 
     $self->{'dump_cb'} = $params{'dump_cb'};
@@ -624,7 +633,7 @@ sub setup_and_start_dump {
 	    (my $err = $self->{'scribe'}->check_data_path($params{'data_path'}))) {
 	    return $params{'dump_cb'}->(
 		result => "FAILED",
-		device_errors => [ ['error', "$err"] ],
+		device_errors => [ 'error', "$err" ],
 		size => 0,
 		duration => 0.0,
 		total_duration => 0);
@@ -634,7 +643,7 @@ sub setup_and_start_dump {
 
     step process_args => sub {
 	# extract the splitting-related parameters, stripping out empty strings
-	my %splitting_args = map {
+	%splitting_args = map {
 	    (defined $params{$_} && $params{$_} ne '')? ($_, $params{$_}) : ()
 	} qw(
 	    dle_tape_splitsize dle_split_diskbuffer dle_fallback_splitsize dle_allow_split
@@ -647,6 +656,18 @@ sub setup_and_start_dump {
 		if (exists $splitting_args{$_});
 	}
 
+	# we've started the xfer now, but the destination won't actually write
+	# any data until we call start_dump.  And we'll need a device for that.
+
+	$self->{'scribe'}->wait_device(finished_cb => $steps->{'got_device'});
+    };
+
+    step got_device => sub {
+	my ($err) = @_;
+
+	if ($err) {
+	    debug("got_device failed: $err");
+	}
 	my $device = $self->{'scribe'}->get_device();
 	if (!defined $device) {
 	    confess "no device is available to create an xfer_dest";
@@ -703,7 +724,7 @@ sub setup_and_start_dump {
 	    }
         });
 
-	# we've started the xfer now, but the destination won't actually write
+	# we've found a device, but the destination won't actually write
 	# any data until we call start_dump.  And we'll need a header for that.
 
 	$steps->{'get_header'}->();
