@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998, 2000 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -2862,7 +2862,8 @@ start_processing_data(
     }
 
     /* decrypt */
-    if (ctl_data->file.encrypted) {
+    if (ctl_data->file.encrypted &&
+	am_has_feature(tapesrv_features, fe_amrecover_receive_unfiltered)) {
 	char *argv[3];
 	int  crypt_out;
 	int  errfd = fileno(stderr);
@@ -2876,7 +2877,8 @@ start_processing_data(
     }
 
     /* decompress */
-    if (ctl_data->file.compressed) {
+    if (ctl_data->file.compressed &&
+	am_has_feature(tapesrv_features, fe_amrecover_receive_unfiltered)) {
 	char *argv[3];
 	int  comp_out;
 	int  errfd = fileno(stderr);
