@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -53,6 +53,8 @@ static void estimate(int argc, char **argv);
 static void estimate_one(disk_t *dp);
 void force(int argc, char **argv);
 void force_one(disk_t *dp);
+void force_level_1(int argc, char **argv);
+void force_level_1_one(disk_t *dp);
 void unforce(int argc, char **argv);
 void unforce_one(disk_t *dp);
 void force_bump(int argc, char **argv);
@@ -111,6 +113,8 @@ static const struct {
 	T_(" [<hostname> [<disks>]* ]*\t# Print server estimate.") },
     { "force", force,
 	T_(" [<hostname> [<disks>]* ]+\t\t# Force level 0 at next run.") },
+    { "force-level-1", force_level_1,
+	T_(" [<hostname> [<disks>]* ]+\t\t# Force level 1 at next run.") },
     { "unforce", unforce,
 	T_(" [<hostname> [<disks>]* ]+\t# Clear force command.") },
     { "force-bump", force_bump,
@@ -229,13 +233,15 @@ main(
     if(argc < 3) usage();
 
     set_config_overrides(cfg_ovr);
-    config_init(CONFIG_INIT_EXPLICIT_NAME, argv[1]);
 
     if(strcmp(argv[2],"version") == 0) {
+	config_init(0, NULL);
 	show_version(argc, argv);
 	goto done;
     }
 
+    config_init(CONFIG_INIT_EXPLICIT_NAME, argv[1]);
+
     conf_diskfile = config_dir_relative(getconf_str(CNF_DISKFILE));
     read_diskfile(conf_diskfile, &diskq);
     amfree(conf_diskfile);
@@ -285,7 +291,9 @@ main(
 
 done:
 
-    free_disklist(&diskq);
+    unload_disklist();
+    diskq.head = NULL;
+    diskq.tail = NULL;
     dbclose();
     return 0;
 }
@@ -462,9 +470,14 @@ force_one(
 
     get_info(hostname, diskname, &info);
     SET(info.command, FORCE_FULL);
+    if (ISSET(info.command, FORCE_LEVEL_1)) {
+	CLR(info.command, FORCE_LEVEL_1);
+	g_printf(_("%s: WARNING: %s:%s FORCE-LEVEL-1 command was cleared.\n"),
+	       get_pname(), hostname, diskname);
+    }
     if (ISSET(info.command, FORCE_BUMP)) {
 	CLR(info.command, FORCE_BUMP);
-	g_printf(_("%s: WARNING: %s:%s FORCE_BUMP command was cleared.\n"),
+	g_printf(_("%s: WARNING: %s:%s FORCE-BUMP command was cleared.\n"),
 	       get_pname(), hostname, diskname);
     }
     if(put_info(hostname, diskname, &info) == 0) {
@@ -494,6 +507,48 @@ force(
 /* ----------------------------------------------- */
 
 
+void
+force_level_1_one(
+    disk_t *	dp)
+{
+    char *hostname = dp->host->hostname;
+    char *diskname = dp->name;
+    info_t info;
+
+    get_info(hostname, diskname, &info);
+    SET(info.command, FORCE_LEVEL_1);
+    if (ISSET(info.command, FORCE_FULL)) {
+	CLR(info.command, FORCE_FULL);
+	g_printf(_("%s: WARNING: %s:%s FORCE command was cleared.\n"),
+	       get_pname(), hostname, diskname);
+    }
+    if (ISSET(info.command, FORCE_BUMP)) {
+	CLR(info.command, FORCE_BUMP);
+	g_printf(_("%s: WARNING: %s:%s FORCE-BUMP command was cleared.\n"),
+	       get_pname(), hostname, diskname);
+    }
+    if(put_info(hostname, diskname, &info) == 0) {
+	g_printf(_("%s: %s:%s is set to a forced level 1 at next run.\n"),
+		 get_pname(), hostname, diskname);
+    } else {
+	g_fprintf(stderr, _("%s: %s:%s could not be forced.\n"),
+		get_pname(), hostname, diskname);
+    }
+}
+
+
+void
+force_level_1(
+    int		argc,
+    char **	argv)
+{
+    diskloop(argc, argv, "force", force_level_1_one);
+}
+
+
+/* ----------------------------------------------- */
+
+
 void
 unforce_one(
     disk_t *	dp)
@@ -513,6 +568,16 @@ unforce_one(
 		    _("%s: force command for %s:%s could not be cleared.\n"),
 		    get_pname(), hostname, diskname);
 	}
+    } else if (ISSET(info.command, FORCE_LEVEL_1)) {
+	CLR(info.command, FORCE_LEVEL_1);
+	if(put_info(hostname, diskname, &info) == 0){
+	    g_printf(_("%s: force-level-1 command for %s:%s cleared.\n"),
+		   get_pname(), hostname, diskname);
+	} else {
+	    g_fprintf(stderr,
+		    _("%s: force-level-1 command for %s:%s could not be cleared.\n"),
+		    get_pname(), hostname, diskname);
+	}
     }
     else {
 	g_printf(_("%s: no force command outstanding for %s:%s, unchanged.\n"),
@@ -544,12 +609,17 @@ force_bump_one(
     SET(info.command, FORCE_BUMP);
     if (ISSET(info.command, FORCE_NO_BUMP)) {
 	CLR(info.command, FORCE_NO_BUMP);
-	g_printf(_("%s: WARNING: %s:%s FORCE_NO_BUMP command was cleared.\n"),
+	g_printf(_("%s: WARNING: %s:%s FORCE-NO-BUMP command was cleared.\n"),
 	       get_pname(), hostname, diskname);
     }
     if (ISSET(info.command, FORCE_FULL)) {
 	CLR(info.command, FORCE_FULL);
-	g_printf(_("%s: WARNING: %s:%s FORCE_FULL command was cleared.\n"),
+	g_printf(_("%s: WARNING: %s:%s FORCE command was cleared.\n"),
+	       get_pname(), hostname, diskname);
+    }
+    if (ISSET(info.command, FORCE_LEVEL_1)) {
+	CLR(info.command, FORCE_LEVEL_1);
+	g_printf(_("%s: WARNING: %s:%s FORCE-LEVEL-1 command was cleared.\n"),
 	       get_pname(), hostname, diskname);
     }
     if(put_info(hostname, diskname, &info) == 0) {
@@ -586,7 +656,7 @@ force_no_bump_one(
     SET(info.command, FORCE_NO_BUMP);
     if (ISSET(info.command, FORCE_BUMP)) {
 	CLR(info.command, FORCE_BUMP);
-	g_printf(_("%s: WARNING: %s:%s FORCE_BUMP command was cleared.\n"),
+	g_printf(_("%s: WARNING: %s:%s FORCE-BUMP command was cleared.\n"),
 	       get_pname(), hostname, diskname);
     }
     if(put_info(hostname, diskname, &info) == 0) {
