@@ -1,5 +1,5 @@
 #! @PERL@
-# Copyright (c) 2009-2012 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -90,7 +90,7 @@ sub _user_msg_fn {
             print STDERR "Searching for label '$params{'label'}':";
         } elsif (exists($params{'slot_result'}) ||
                  exists($params{'search_result'})) {
-            if (defined($params{'err'})) {
+            if (defined($params{'err'}) and ref($params{'err'}) eq "HASH") {
                 if (exists($params{'search_result'}) &&
                     defined($params{'err'}->{'this_slot'})) {
                     print STDERR "slot $params{'err'}->{'this_slot'}:";
@@ -98,18 +98,38 @@ sub _user_msg_fn {
                 print STDERR " $params{'err'}\n";
             } elsif (!$params{'res'}) {
                 my $volume_label = $params{'label'};
+		if ($params{'slot'}) {
+		    print STDERR "slot $params{'slot'}:";
+		}
                 if ($params{'active'}) {
                     print STDERR " volume '$volume_label' is still active and cannot be overwritten\n";
                 } elsif ($params{'does_not_match_labelstr'}) {
                     print STDERR " volume '$volume_label' does not match labelstr '$params{'labelstr'}'\n";
                 } elsif ($params{'not_in_tapelist'}) {
                     print STDERR " volume '$volume_label' is not in the tapelist\n"
+		} elsif ($params{'non_amanda'}) {
+		    print STDERR " not an amanda volume\n";
+		} elsif ($params{'volume_error'}) {
+		    print STDERR " $params{'err'}\n";
+		} elsif ($params{'not_autolabel'}) {
+		    print STDERR " The volume can't be labelled\n";
+		} elsif ($params{'empty'}) {
+		    print STDERR " The volume is empty\n";
+		} elsif ($params{'not_success'}) {
+		    print STDERR " $params{'err'}\n";
                 } else {
                     print STDERR " volume '$volume_label'\n";
                 }
             } else { # res must be defined
+		my $directtcp = "";
                 my $res = $params{'res'};
                 my $dev = $res->{'device'};
+
+		if ($dev->directtcp_supported()) {
+		    $directtcp = "DIRECTTCP ";
+		}
+		print STDOUT "DATA-PATH AMANDA $directtcp\n";
+
                 if (exists($params{'search_result'})) {
                     print STDERR "found in slot $res->{'this_slot'}:";
                 }
