@@ -1,5 +1,5 @@
 #! @PERL@
-# Copyright (c) 2010-2012 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2010-2013 Zmanda Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -91,8 +91,9 @@ my $timestamp = strftime "%Y%m%d%H%M%S", @now;
 my $datestamp = strftime "%Y%m%d", @now;
 my $starttime_locale_independent = strftime "%Y-%m-%d %H:%M:%S %Z", @now;
 my $trace_log_filename = "$logdir/log";
-my $amdump_log_filename_default = "$logdir/amdump";
-my $amdump_log_filename = "$logdir/amdump.$timestamp";
+my $amdump_log_pathname_default = "$logdir/amdump";
+my $amdump_log_pathname = "$logdir/amdump.$timestamp";
+my $amdump_log_filename = "amdump.$timestamp";
 my $exit_code = 0;
 my $amdump_log = \*STDERR;
 
@@ -128,12 +129,7 @@ sub run_subprocess {
     my $s = $? >> 8;
     debug("$proc exited with code $s");
     if ($?) {
-	if ($exit_code == 0) {
-	    debug("ignoring failing exit code $s from $proc");
-	} else {
-	    debug("recording failing exit code $s from $proc for amdump exit");
-	    $exit_code = $s;
-	}
+	$exit_code |= $s;
     }
 }
 
@@ -148,7 +144,7 @@ sub wait_for_hold {
 }
 
 sub bail_already_running {
-    my $msg = "An Amanda process is already running - please run amcleanup manually";
+    my $msg = "An Amanda process is already running - please run amcleanup if you wish to abort the current process and clean up open log files";
     debug($msg);
     amdump_log($msg);
 
@@ -170,7 +166,7 @@ EOF
 }
 
 sub do_amcleanup {
-    return unless -f $amdump_log_filename_default || -f $trace_log_filename;
+    return unless -f $amdump_log_pathname_default || -f $trace_log_filename;
 
     # logfiles are still around.  First, try an amcleanup -p to see if
     # the actual processes are already dead
@@ -178,7 +174,7 @@ sub do_amcleanup {
     run_subprocess("$sbindir/amcleanup", '-p', $config_name, @config_overrides_opts);
 
     # and check again
-    return unless -f $amdump_log_filename_default || -f $trace_log_filename;
+    return unless -f $amdump_log_pathname_default || -f $trace_log_filename;
 
     bail_already_running();
 }
@@ -204,10 +200,10 @@ sub start_logfiles {
     debug("beginning amdump log");
     $amdump_log = undef;
     # Must be opened in append so that all subprocess can write to it.
-    open($amdump_log, ">>", $amdump_log_filename)
-	or die("could not open amdump log file '$amdump_log_filename': $!");
-    unlink $amdump_log_filename_default;
-    symlink $amdump_log_filename, $amdump_log_filename_default;
+    open($amdump_log, ">>", $amdump_log_pathname)
+	or die("could not open amdump log file '$amdump_log_pathname': $!");
+    unlink $amdump_log_pathname_default;
+    symlink $amdump_log_filename, $amdump_log_pathname_default;
 }
 
 sub planner_driver_pipeline {
@@ -299,8 +295,8 @@ sub trim_indexes {
 sub roll_amdump_logs {
     debug("renaming amdump log and trimming old amdump logs (beyond tapecycle+2)");
 
-    unlink "$amdump_log_filename_default.1";
-    rename $amdump_log_filename_default, "$amdump_log_filename_default.1";
+    unlink "$amdump_log_pathname_default.1";
+    rename $amdump_log_pathname_default, "$amdump_log_pathname_default.1";
 
     # keep the latest tapecycle files.
     my @files = sort {-M $b <=> -M $a} grep { !/^\./ && -f "$_"} <$logdir/amdump.*>;
