@@ -1,5 +1,5 @@
 #! @PERL@
-# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,7 +30,7 @@ use IPC::Open3;
 use Amanda::Device qw( :constants );
 use Amanda::Debug qw( :logging );
 use Amanda::Config qw( :init :getconf config_dir_relative );
-use Amanda::Util qw( :constants );
+use Amanda::Util qw( :constants :quoting );
 use Amanda::Changer;
 use Amanda::Constants;
 use Amanda::MainLoop;
@@ -53,6 +53,10 @@ use vars qw( @ISA );
 sub new {
     my $class = shift;
 
+    if (!-r STDIN) {
+	return undef;
+    }
+
     my $self = {
 	input_src => undef};
     return bless ($self, $class);
@@ -175,7 +179,10 @@ GetOptions(
     'extract' => \$opt_extract,
     'directory=s' => \$opt_directory,
     'data-path=s' => \$opt_data_path,
-    'application-property=s' => \%application_property,
+    'application-property=s' => sub {
+	    my ($name, $value) = split '=', $_[1];
+	    push @{$application_property{$name}}, $value;
+	},
     'exact-match' => \$opt_exact_match,
     'b=s' => \$opt_blocksize,
     'd=s' => \$opt_device,
@@ -327,6 +334,10 @@ if ($cfgerr_level >= $CFGERR_WARNINGS) {
 
 Amanda::Util::finish_setup($RUNNING_AS_DUMPUSER);
 
+if (!$opt_device && getconf_linenum($CNF_TPCHANGER) == -2) {
+    $opt_device = getconf($CNF_TPCHANGER);
+}
+
 my $exit_status = 0;
 my $clerk;
 use Data::Dumper;
@@ -391,9 +402,14 @@ sub main {
     my %recovery_params;
     my $timer;
     my $is_tty;
+    my $last_is_size;
     my $delay;
     my $directtcp = 0;
     my @directtcp_command;
+    my $xfer_dest;
+    my $xfer_app;
+    my $app_success;
+    my $app_error;
 
     my $steps = define_steps
 	cb_ref => \$finished_cb;
@@ -492,6 +508,9 @@ sub main {
 
 	$recovery_done = 0;
 	%recovery_params = ();
+	$app_success = 0;
+	$app_error = 0;
+	$xfer_app = undef;
 
 	$clerk->get_xfer_src(
 	    dump => $current_dump,
@@ -504,11 +523,17 @@ sub main {
 
 	my $dle_str = $hdr->{'dle_str'};
 	my $p1 = XML::Simple->new();
-	my $dle = $p1->XMLin($dle_str);
+	my $dle;
+	if (defined $dle_str) {
+	    eval { $dle = $p1->XMLin($dle_str); };
+	    if ($@) {
+		print "ERROR: XML error\n";
+		debug("XML Error: $@\n$dle_str");
+	    }
+	}
 
 	# and set up the destination..
 	my $dest_fh;
-	my $xfer_dest;
 	my @filters;
 
 	if (defined $opt_data_path and $opt_data_path eq 'directtcp' and !$directtcp_supported) {
@@ -536,7 +561,7 @@ sub main {
 		    return failure("Unknown program '$program' in header; no validation to perform",
 				   $finished_cb);
 		}
-		@argv = $validation_programs{$program};
+		@argv = @{$validation_programs{$program}};
 	    } else {
 		if (!defined $hdr->{application}) {
 		    return failure("Application not set", $finished_cb);
@@ -572,18 +597,60 @@ sub main {
 		push @argv, "--level", $hdr->{'dumplevel'};
 		push @argv, "--directory", $opt_directory;
 
-		# add application_property
-		while (my($name, $value) = each(%application_property)) {
-		    push @argv, "--".$name, $value if $value;
-		}
+		if ($bsu->{'recover-dump-state-file'}) {
+		    my $host = sanitise_filename("".$hdr->{'name'});
+		    my $disk = sanitise_filename("".$hdr->{'disk'});
+		    my $state_filename = getconf($CNF_INDEXDIR) . '/' . $host .
+				 '/' . $disk . '/' . $hdr->{'datestamp'} . '_' .
+				 $hdr->{'dumplevel'} . '.state';
 
-		#merge property from header;
-		while (my($name, $value) = each (%{$dle->{'backup-program'}->{'property'}})) {
-		    if (!exists $application_property{$name}) {
-			push @argv, "--".$name, $value->{'value'};
+		    if (-e $state_filename) {
+			push @argv, "--recover-dump-state-file",
+				    $state_filename;
 		    }
 		}
 
+		# add application_property
+		while (my($name, $values) = each(%application_property)) {
+		    foreach my $value (@{$values}) {
+			push @argv, "--".$name, $value if defined $value;
+		    }
+		}
+
+		#merge property from header;
+		if (exists $dle->{'backup-program'}->{'property'}->{'name'} and
+		    !UNIVERSAL::isa($dle->{'backup-program'}->{'property'}->{'name'}, "HASH")) {
+		    # header have one property
+		    my $name = $dle->{'backup-program'}->{'property'}->{'name'};
+		    if (!exists $application_property{$name}) {
+			my $values = $dle->{'backup-program'}->{'property'}->{'value'};
+			if (UNIVERSAL::isa( $values, "ARRAY" )) {
+			    # multiple values
+			    foreach my $value (@{$values}) {
+				push @argv, "--".$name, $value if defined $value;
+			    }
+			} else {
+			    # one value
+			    push @argv, "--".$name, $values;
+			}
+		    }
+		} elsif (exists $dle->{'backup-program'}->{'property'}) {
+		    # header have multiple properties
+		    while (my($name, $values) =
+			 each (%{$dle->{'backup-program'}->{'property'}})) {
+			if (!exists $application_property{$name}) {
+			    if (UNIVERSAL::isa( $values->{'value'}, "ARRAY" )) {
+				# multiple values
+				foreach my $value (@{$values->{'value'}}) {
+				    push @argv, "--".$name, $value if defined $value;
+				}
+			    } else {
+				# one value
+				push @argv, "--".$name, $values->{'value'};
+			    }
+			}
+		    }
+		}
 	    }
 	    $directtcp = $directtcp_supported;
 	    if ($directtcp_supported) {
@@ -593,10 +660,10 @@ sub main {
 		# set up the extraction command as a filter element, since
 		# we need its stderr.
 		debug("Running: ". join(' ',@argv));
-		push @filters, Amanda::Xfer::Filter::Process->new(\@argv, 0);
+		$xfer_app =  Amanda::Xfer::Filter::Process->new(\@argv, 0, 0, 1, 1);
 
-		$dest_fh = \*STDOUT;
-		$xfer_dest = Amanda::Xfer::Dest::Fd->new($dest_fh);
+		#$dest_fh = \*STDOUT;
+		$xfer_dest = Amanda::Xfer::Dest::Buffer->new(1048576);
 	    }
 	} elsif ($opt_pipe) {
 	    $dest_fh = \*STDOUT;
@@ -627,6 +694,10 @@ sub main {
 	$timer->set_callback(sub {
 	    my $size = $xfer_src->get_bytes_read();
 	    if ($is_tty) {
+		if (!$last_is_size) {
+		    print STDERR "\n";
+		    $last_is_size = 1;
+		}
 		print STDERR "\r" . int($size/1024) . " kb ";
 	    } else {
 		print STDERR "READ SIZE: " . int($size/1024) . " kb\n";
@@ -640,11 +711,11 @@ sub main {
 	    if ($hdr->{'srv_encrypt'}) {
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
-			[ $hdr->{'srv_encrypt'}, $hdr->{'srv_decrypt_opt'} ], 0);
+			[ $hdr->{'srv_encrypt'}, $hdr->{'srv_decrypt_opt'} ], 0, 0, 0, 1);
 	    } elsif ($hdr->{'clnt_encrypt'}) {
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
-			[ $hdr->{'clnt_encrypt'}, $hdr->{'clnt_decrypt_opt'} ], 0);
+			[ $hdr->{'clnt_encrypt'}, $hdr->{'clnt_decrypt_opt'} ], 0, 0, 0, 1);
 	    } else {
 		return failure("could not decrypt encrypted dump: no program specified",
 			    $finished_cb);
@@ -672,17 +743,17 @@ sub main {
 		# TODO: this assumes that srvcompprog takes "-d" to decompress
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
-			[ $hdr->{'srvcompprog'}, "-d" ], 0);
+			[ $hdr->{'srvcompprog'}, "-d" ], 0, 0, 0, 1);
 	    } elsif ($hdr->{'clntcompprog'}) {
 		# TODO: this assumes that clntcompprog takes "-d" to decompress
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
-			[ $hdr->{'clntcompprog'}, "-d" ], 0);
+			[ $hdr->{'clntcompprog'}, "-d" ], 0, 0, 0, 1);
 	    } else {
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
 			[ $Amanda::Constants::UNCOMPRESS_PATH,
-			  $Amanda::Constants::UNCOMPRESS_OPT ], 0);
+			  $Amanda::Constants::UNCOMPRESS_OPT ], 0, 0, 0, 1);
 	    }
 
 	    # adjust the header
@@ -697,7 +768,7 @@ sub main {
 	    push @filters,
 		Amanda::Xfer::Filter::Process->new(
 		    [ $Amanda::Constants::COMPRESS_PATH,
-		      $compress_opt ], 0);
+		      $compress_opt ], 0, 0, 0, 1);
 
 	    # adjust the header
 	    $hdr->{'compressed'} = 1;
@@ -706,6 +777,10 @@ sub main {
 	    $hdr->{'comp_suffix'} = $Amanda::Constants::COMPRESS_SUFFIX;
 	}
 
+	if ($xfer_app) {
+	    push @filters, $xfer_app;
+	}
+
 	# write the header to the destination if requested
 	$hdr->{'blocksize'} = Amanda::Holding::DISK_BLOCK_BYTES;
 	if (defined $opt_header or defined $opt_header_file or defined $opt_header_fd) {
@@ -747,7 +822,15 @@ sub main {
 			my $line = $buffer;
 			chomp $line;
 			if (length($line) > 1) {
-			    print STDERR "filter stderr: $line\n";
+			    if (!$app_success || $app_error) {
+				if ($is_tty) {
+				    if ($last_is_size) {
+					print STDERR "\n";
+					$last_is_size = 0;
+				    }
+				}
+				print STDERR "filter stderr: $line\n";
+			    }
 			    debug("filter stderr: $line");
 			}
 			$buffer = "";
@@ -818,7 +901,21 @@ sub main {
     step handle_xmsg => sub {
 	my ($src, $msg, $xfer) = @_;
 
+	if ($msg->{'elt'} == $xfer_app) {
+	    if ($msg->{'type'} == $XMSG_DONE) {
+		$app_success = 1;
+	    } elsif ($msg->{'type'} == $XMSG_ERROR) {
+	    } elsif ($msg->{'type'} == $XMSG_INFO and
+		$msg->{'message'} eq "SUCCESS") {
+		$app_success = 1;
+	    } elsif ($msg->{'type'} == $XMSG_INFO and
+		$msg->{'message'} eq "ERROR") {
+		$app_error = 1;
+	    }
+	}
+
 	$clerk->handle_xmsg($src, $msg, $xfer);
+
 	if ($msg->{'type'} == $XMSG_INFO) {
 	    Amanda::Debug::info($msg->{'message'});
 	} elsif ($msg->{'type'} == $XMSG_ERROR) {
@@ -835,10 +932,23 @@ sub main {
 
     step filter_done => sub {
 	if ($is_tty) {
-	    print STDERR "\r" . int($recovery_params{'bytes_read'}/1024) . " kb ";
+	    if (!$last_is_size) {
+		print STDERR "\n";
+	    }
+	    $last_is_size = 0;
+	    print STDERR "\r" . int($recovery_params{'bytes_read'}/1024) . " kb \n";
 	} else {
 	    print STDERR "READ SIZE: " . int($recovery_params{'bytes_read'}/1024) . " kb\n";
 	}
+	if ($xfer_dest && $xfer_dest->isa("Amanda::Xfer::Dest::Buffer")) {
+	    my $buf = $xfer_dest->get();
+	    my @lines = split "\n", $buf;
+	    foreach (@lines) {
+		next if $_ =~ /^$/;
+		print STDERR "Application stdout: $_\n";
+		debug("Application stdout: $_");
+	    }
+	}
 	@xfer_errs = (@xfer_errs, @{$recovery_params{'errors'}})
 	    if $recovery_params{'errors'};
 	return failure(join("; ", @xfer_errs), $finished_cb)
@@ -864,7 +974,7 @@ sub main {
 	    $timer->remove();
 	    $timer = undef;
 	}
-	print STDERR "\n" if $is_tty;
+	print STDERR "\n" if $is_tty and $last_is_size;
 	return failure($err, $finished_cb) if $err;
 
 	$finished_cb->();
