@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -188,6 +188,8 @@ main(
 
     dbrename(get_config_name(), DBG_SUBDIR_SERVER);
 
+    holding_cleanup(NULL, stdout);
+
     /* load DLEs from the holding disk, in case there's anything to flush there */
     search_holding_disk(&holding_files, &holding_disklist);
     /* note that the dumps are added to the global disklist, so we need not
@@ -225,7 +227,7 @@ main(
     }
     if (access(conf_logfile, F_OK) == 0) {
 	char *process_name = get_master_process(conf_logfile);
-	error(_("%s exists: %s is already running, or you must run amcleanup"), conf_logfile, process_name);
+	error(_("%s exists: %s is already running, please run amcleanup if you wish to abort the current process and clean up open log files"), conf_logfile, process_name);
 	/*NOTREACHED*/
     }
 
