@@ -1,5 +1,5 @@
 #! @PERL@
-# Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2010-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -516,7 +516,7 @@ sub xfer_src_cb {
 	if ($header->{'srv_encrypt'}) {
 	    push @filters,
 		Amanda::Xfer::Filter::Process->new(
-		    [ $header->{'srv_encrypt'}, $header->{'srv_decrypt_opt'} ], 0);
+		    [ $header->{'srv_encrypt'}, $header->{'srv_decrypt_opt'} ], 0, 0, 0, 1);
 	    $header->{'encrypted'} = 0;
 	    $header->{'srv_encrypt'} = '';
 	    $header->{'srv_decrypt_opt'} = '';
@@ -528,7 +528,7 @@ sub xfer_src_cb {
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
 		        [ $header->{'clnt_encrypt'},
-			  $header->{'clnt_decrypt_opt'} ], 0);
+			  $header->{'clnt_decrypt_opt'} ], 0, 0, 0, 1);
 		$header->{'encrypted'} = 0;
 		$header->{'srv_encrypt'} = '';
 		$header->{'srv_decrypt_opt'} = '';
@@ -553,7 +553,7 @@ sub xfer_src_cb {
 	    # TODO: this assumes that srvcompprog takes "-d" to decrypt
 	    push @filters,
 		Amanda::Xfer::Filter::Process->new(
-		    [ $header->{'srvcompprog'}, "-d" ], 0);
+		    [ $header->{'srvcompprog'}, "-d" ], 0, 0, 0, 1);
 	    # adjust the header
 	    $header->{'compressed'} = 0;
 	    $header->{'uncompress_cmd'} = '';
@@ -563,7 +563,7 @@ sub xfer_src_cb {
 		# TODO: this assumes that clntcompprog takes "-d" to decrypt
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
-			[ $header->{'clntcompprog'}, "-d" ], 0);
+			[ $header->{'clntcompprog'}, "-d" ], 0, 0, 0, 1);
 		# adjust the header
 		$header->{'compressed'} = 0;
 		$header->{'uncompress_cmd'} = '';
@@ -578,7 +578,7 @@ sub xfer_src_cb {
 		push @filters,
 		    Amanda::Xfer::Filter::Process->new(
 			[ $Amanda::Constants::UNCOMPRESS_PATH,
-			  $Amanda::Constants::UNCOMPRESS_OPT ], 0);
+			  $Amanda::Constants::UNCOMPRESS_OPT ], 0, 0, 0, 1);
 		# adjust the header
 		$header->{'compressed'} = 0;
 		$header->{'uncompress_cmd'} = '';
@@ -890,7 +890,8 @@ sub getline {
 	last if $c eq "\n";
     }
 
-    my $chopped = $line;
+    $line =~ /^(.*)$/;
+    my $chopped = $1;
     $chopped =~ s/[\r\n]*$//g;
     debug("CTL << $chopped");
 
@@ -914,8 +915,8 @@ sub getline_async {
 	return $async_read_cb->($err, undef) if $err;
 
 	$buf .= $data;
-	if ($buf =~ /\r\n$/) {
-	    my $chopped = $buf;
+	if ($buf =~ /^(.*\r\n)$/) {
+	    my $chopped = $1;
 	    $chopped =~ s/[\r\n]*$//g;
 	    debug("CTL << $chopped");
 
