@@ -1,5 +1,5 @@
 #! @PERL@
-# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -224,6 +224,9 @@ sub {
 	    } elsif ($err->empty and defined $err->{'slot'}) {
 		$last_slot = $err->{'slot'};
 	        print STDERR sprintf("slot %3s: empty\n", $last_slot);
+	    } elsif ($err->invalid and defined $err->{'slot'}) {
+		$last_slot = $err->{'slot'};
+	        print STDERR sprintf("slot %3s: %s\n", $last_slot, "$err");
 	    } else {
 		return failure($err, $finished_cb) if $err;
 	    }
@@ -303,10 +306,10 @@ sub {
 	for my $sl (@$inv) {
 	    my $line = "slot $sl->{slot}:";
 	    my $tle;
-	    if (!defined($sl->{device_status}) && !defined($sl->{label})) {
-		$line .= " unknown state";
-	    } elsif ($sl->{'state'} == Amanda::Changer::SLOT_EMPTY) {
+	    if ($sl->{'state'} == Amanda::Changer::SLOT_EMPTY) {
 		$line .= " empty";
+	    } elsif (!defined($sl->{device_status}) && !defined($sl->{label})) {
+		$line .= " unknown state";
 	    } else {
 		if (defined $sl->{label}) {
 		    $line .= " label $sl->{label}";
@@ -363,6 +366,40 @@ sub {
     $chg->inventory(inventory_cb => $inventory_cb);
 });
 
+subcommand("verify", "verify", "verify the changer is correctly configured",
+sub {
+    my ($finished_cb, @args) = @_;
+
+    my $chg = load_changer($finished_cb) or return;
+
+    if (@args != 0) {
+	return usage($finished_cb);
+    }
+
+    # TODO -- support an --xml option
+
+    my $verify_cb = make_cb(verify => sub {
+	my ($err, @results) = @_;
+	if ($err) {
+	    if ($err->notimpl) {
+		if ($err->{'message'}) {
+		    print STDERR "verify not supported by this changer: $err->{'message'}\n";
+		} else {
+		    print STDERR "verify not supported by this changer\n";
+		}
+	    } else {
+		print STDERR "$err\n";
+	    }
+	} else {
+	    print STDERR join("\n", @results);
+	    print STDERR "\n";
+	}
+	$chg->quit();
+	return $finished_cb->();
+    });
+    $chg->verify(finished_cb => $verify_cb);
+});
+
 subcommand("current", "current", "load and show the contents of the current slot",
 sub {
     my ($finished_cb, @args) = @_;
@@ -539,7 +576,7 @@ sub {
 		    print STDERR "slot $params{'err'}->{'slot'}:";
 		}
 		print STDERR " $params{'err'}\n";
-	    } else { # res must be defined
+	    } elsif ($params{'res'}) {
 		my $res = $params{'res'};
 		my $dev = $res->{'device'};
 		if (exists($params{'search_result'})) {
@@ -552,7 +589,9 @@ sub {
 		    } elsif ($params{'does_not_match_labelstr'}) {
 			print STDERR " volume '$volume_label' does not match labelstr '$params{'labelstr'}'\n";
 		    } elsif ($params{'not_in_tapelist'}) {
-			print STDERR " volume '$volume_label' is not in the tapelist\n"
+			print STDERR " volume '$volume_label' is not in the tapelist\n";
+		    } elsif ($params{'relabeled'}) {
+			print STDERR " volume '$volume_label' from another config will be relabeled\n";
 		    } else {
 			print STDERR " volume '$volume_label'\n";
 		    }
@@ -609,8 +648,10 @@ sub {
 
 	my $modestr = ($mode == $ACCESS_APPEND)? "append" : "write";
 	my $slot = $res->{'this_slot'};
-	if (defined $res->{'device'} and defined $res->{'device'}->volume_label()) {
+	if (defined $res->{'device'} and defined $res->{'device'}->volume_label() and $res->{'device'}->volume_label() eq $label) {
 	    print STDERR "Will $modestr to volume '$label' in slot $slot.\n";
+	} elsif (defined $res->{'device'} and defined $res->{'device'}->volume_label()) {
+	    print STDERR "Will $modestr label '$label' to '" . $res->{'device'}->volume_label() . "' labelled volume in slot $slot.\n";
 	} else {
 	    my $header = $res->{'device'}->volume_header();
 	    if ($header->{'type'} == $Amanda::Header::F_WEIRD) {
