@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -213,7 +213,9 @@ main(
     amfree(olddir);
     amfree(conf_logdir);
     clear_tapelist();
-    free_disklist(&diskl);
+    unload_disklist();
+    diskl.head = NULL;
+    diskl.tail = NULL;
 
     dbclose();
 
