@@ -1,9 +1,10 @@
 #! @PERL@
-# Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2008-2013 Zmanda, Inc.  All Rights Reserved.
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published
-# by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -30,6 +31,10 @@ use vars qw( @ISA );
 sub new {
     my $class = shift;
 
+    if (!-r STDIN) {
+	return undef;
+    }
+
     my $self = {
 	input_src => undef};
     return bless ($self, $class);
@@ -93,6 +98,7 @@ sub user_request {
 package Amvault;
 
 use Amanda::Config qw( :getconf config_dir_relative );
+use Amanda::Disklist;
 use Amanda::Debug qw( :logging debug );
 use Amanda::Xfer qw( :constants );
 use Amanda::Header qw( :constants );
@@ -123,6 +129,8 @@ sub new {
     bless {
 	quiet => $params{'quiet'},
 	fulls_only => $params{'fulls_only'},
+	latest_fulls => $params{'latest_fulls'},
+	incrs_only => $params{'incrs_only'},
 	opt_export => $params{'opt_export'},
 	opt_dumpspecs => $params{'opt_dumpspecs'},
 	opt_dry_run => $params{'opt_dry_run'},
@@ -184,7 +192,7 @@ sub do_amcleanup {
 
 sub bail_already_running() {
     my $self = shift;
-    my $msg = "An Amanda process is already running - please run amcleanup manually";
+    my $msg = "An Amanda process is already running - please run amcleanup if you wish to abort the current process and clean up open log files";
     print "$msg\n";
     debug($msg);
     $self->{'exit_cb'}->(1);
@@ -194,6 +202,14 @@ sub run {
     my $self = shift;
     my ($exit_cb) = @_;
 
+    $self->{'is_tty'} = -t STDERR;
+    if ($self->{'is_tty'}) {
+	$self->{'delay'} = 1000; # 1 second
+    } else {
+	$self->{'delay'} = 15000; # 15 seconds
+    }
+    $self->{'last_is_size'} = 0;
+
     die "already called" if $self->{'exit_cb'};
     $self->{'exit_cb'} = $exit_cb;
 
@@ -263,12 +279,15 @@ sub setup_src {
 	$self->vlog("Using latest timestamp: $ts");
     }
 
-    # we need to combine fulls_only, src_write_timestamp, and the set
-    # of dumpspecs.  If they contradict one another, then drop the
+    # we need to combine fulls_only, latest_fulls, incr_only,
+    # src_write_timestamp, and the set of dumpspecs.
+    # If they contradict one another, then drop the
     # non-matching dumpspec with a warning.
     my @dumpspecs;
     if ($self->{'opt_dumpspecs'}) {
-	my $level = $self->{'fulls_only'}? "0" : undef;
+	my $level = $self->{'fulls_only'}? "=0" :
+		    $self->{'latest_fulls'}? "=0" :
+		    $self->{'incrs_only'}? "1-399" : undef;
 	my $swt = $self->{'src_write_timestamp'};
 
 	# filter and adjust the dumpspecs
@@ -286,12 +305,21 @@ sub setup_src {
 	    }
 
 	    if (defined $level) {
-		if (defined $ds_level &&
-		    !match_level($ds_level, $level)) {
-		    $self->vlog("WARNING: dumpspec " . $ds->format() .
-			    " specifies non-full dumps, contradicting --fulls-only;" .
-			    " ignoring dumpspec");
-		    next;
+		if (defined $ds_level) {
+		    if ($self->{'fulls_only'} &&
+			!match_level($ds_level, '0')) {
+			$self->vlog("WARNING: dumpspec " . $ds->format() .
+			    " specifies non-full dumps, contradicting" .
+			    " --fulls-only; ignoring dumpspec");
+			next;
+		    }
+		    if ($self->{'incrs_only'} &&
+			$ds_level eq '0' || $ds_level eq '=0') {
+			$self->vlog("WARNING: dumpspec " . $ds->format() .
+			    " specifies full dumps, contradicting" .
+			    " --incrs-only; ignoring dumpspec");
+			next;
+		    }
 		}
 		$ds_level = $level;
 	    }
@@ -302,7 +330,9 @@ sub setup_src {
 	}
     } else {
 	# convert the timestamp and level to a dumpspec
-	my $level = $self->{'fulls_only'}? "0" : undef;
+	my $level = $self->{'fulls_only'}? "=0" :
+		    $self->{'latest_fulls'}? "=0" :
+		    $self->{'incrs_only'}? "1-399" : undef;
 	push @dumpspecs, Amanda::Cmdline::dumpspec_t->new(
 		undef, undef, undef, $level, $self->{'src_write_timestamp'});
     }
@@ -338,6 +368,7 @@ sub setup_src {
     }
 
     Amanda::Recovery::Planner::make_plan(
+	    latest_fulls => $self->{'latest_fulls'},
 	    dumpspecs => \@dumpspecs,
 	    changer => $src->{'chg'},
 	    plan_cb => sub { $self->plan_cb(@_) });
@@ -352,6 +383,14 @@ sub plan_cb {
 
     $src->{'plan'} = $plan;
 
+    @{$plan->{'dumps'}} = grep {
+	    Amanda::Disklist::get_disk($_->{'hostname'}, $_->{'diskname'})
+	} @{$plan->{'dumps'}};
+    if (!@{$plan->{'dumps'}}) {
+	return $self->failure("No dumps to vault");
+    }
+
+
     if ($self->{'opt_dry_run'}) {
 	my $total_kb = Math::BigInt->new(0);
 
@@ -440,7 +479,7 @@ sub scribe_started {
 
     my $xfers_finished = sub {
 	my ($err) = @_;
-	$self->failure($err) if $err;
+	return $self->failure($err) if $err;
 	$self->quit(0);
     };
 
@@ -460,7 +499,7 @@ sub xfer_dumps {
 	    cb_ref => \$finished_cb;
 
     step get_dump => sub {
-	# reset tracking for teh current dump
+	# reset tracking for the current dump
 	$self->{'current'} = $current = {
 	    src_result => undef,
 	    src_errors => undef,
@@ -504,9 +543,22 @@ sub xfer_dumps {
 	# set up splitting args from the tapetype only, since we have no DLEs
 	my $tt = lookup_tapetype(getconf($CNF_TAPETYPE));
 	sub empty2undef { $_[0]? $_[0] : undef }
+	my $dle_allow_split = 1;
+	my $dle = Amanda::Disklist::get_disk($header->{'name'},
+					     $header->{'disk'});
+	if (defined $dle) {
+	    $dle_allow_split = dumptype_getconf($dle->{'config'}, $DUMPTYPE_ALLOW_SPLIT);
+	}
+	my $xdt_first_dev = $dst->{'scribe'}->get_device();
+	if (!defined $xdt_first_dev) {
+	    return $finished_cb->("no device is available to create an xfer_dest");
+	}
+	my $leom_supported = $xdt_first_dev->property_get("leom");
 	my %xfer_dest_args;
 	if ($tt) {
 	    %xfer_dest_args = get_splitting_args_from_config(
+		dle_allow_split => $dle_allow_split,
+		leom_supported => $leom_supported,
 		part_size_kb =>
 		    empty2undef(tapetype_getconf($tt, $TAPETYPE_PART_SIZE)),
 		part_cache_type_enum =>
@@ -516,8 +568,12 @@ sub xfer_dumps {
 		part_cache_max_size =>
 		    empty2undef(tapetype_getconf($tt, $TAPETYPE_PART_CACHE_MAX_SIZE)),
 	    );
+	} else {
+	    # split only if LEOM is supported.
+	    %xfer_dest_args = get_splitting_args_from_config(
+		dle_allow_split => $dle_allow_split,
+		leom_supported => $leom_supported);
 	}
-	# (else leave %xfer_dest_args empty, for no splitting)
 
 	$xfer_dst = $dst->{'scribe'}->get_xfer_dest(
 	    max_memory => getconf($CNF_DEVICE_OUTPUT_BUFFER_SIZE),
@@ -531,6 +587,20 @@ sub xfer_dumps {
 	$size = $current->{'dump'}->{'bytes'} if exists $current->{'dump'}->{'bytes'};
 	$xfer->start($steps->{'handle_xmsg'}, 0, $size);
 
+	$self->{'timer'} = Amanda::MainLoop::timeout_source($self->{'delay'});
+	$self->{'timer'}->set_callback(sub {
+	    my $size = $dst->{'scribe'}->get_bytes_written();
+	    if ($self->{'is_tty'}) {
+		if (!$self->{'last_is_size'}) {
+		    print STDERR "\n";
+		    $self->{'last_is_size'} = 1;
+		}
+	        print STDERR "\r" . int($size/1024) . " kb ";
+	    } else {
+		debug("WRITTEN SIZE: " . int($size/1024) . " kb");
+	    }
+	});
+
 	# count the "threads" running here (clerk and scribe)
 	$n_threads = 2;
 
@@ -615,12 +685,8 @@ sub xfer_dumps {
 		($logtype == $L_PARTIAL and @errors)? " $msg" : ""));
 	}
 
-	if (@errors) {
-	    return $finished_cb->("transfer failed: " .  join("; ", @errors));
-	} else {
-	    # rinse, wash, and repeat
-	    return $steps->{'get_dump'}->();
-	}
+	# next dump
+	return $steps->{'get_dump'}->();
     };
 }
 
@@ -632,6 +698,15 @@ sub quit {
     my $steps = define_steps
 	    cb_ref => \$exit_cb;
 
+    step quit_timer => sub {
+	if (defined $self->{'timer'}) {
+	    $self->{'timer'}->remove();
+	    $self->{'timer'} = undef;
+	}
+
+	$steps->{'check_exporting'}->();
+    };
+
     # the export may not start until we quit the scribe, so wait for it now..
     step check_exporting => sub {
 	# if we're exporting the final volume, wait for that to complete
@@ -657,7 +732,14 @@ sub quit {
 	$self->{'dst'}{'scan'}->quit();
 	my ($err) = @_;
 	if ($err) {
+	    if ($self->{'is_tty'}) {
+		if ($self->{'last_is_size'}) {
+		    print STDERR "\n";
+		    $self->{'last_is_size'} = 0;
+		}
+	    }
 	    print STDERR "$err\n";
+	    debug("scribe error: $err");
 	    $exit_status = 1;
 	}
 
@@ -677,7 +759,14 @@ sub quit {
     step quit_clerk_finished => sub {
 	my ($err) = @_;
 	if ($err) {
+	    if ($self->{'is_tty'}) {
+		if ($self->{'last_is_size'}) {
+		    print STDERR "\n";
+		    $self->{'last_is_size'} = 0;
+		}
+	    }
 	    print STDERR "$err\n";
+	    debug("clerk error: $err");
 	    $exit_status = 1;
 	}
 
@@ -715,6 +804,12 @@ sub quit {
 sub failure {
     my $self = shift;
     my ($msg) = @_;
+    if ($self->{'is_tty'}) {
+	if ($self->{'last_is_size'}) {
+	    print STDERR "\n";
+	    $self->{'last_is_size'} = 0;
+	}
+    }
     print STDERR "$msg\n";
 
     debug("failure: $msg");
@@ -749,6 +844,9 @@ sub request_volume_permission {
     $params{'perm_cb'}->(allow => 1);
 }
 
+sub scribe_ready {
+}
+
 sub scribe_notif_new_tape {
     my $self = shift;
     my %params = @_;
@@ -764,6 +862,12 @@ sub scribe_notif_new_tape {
     } else {
 	$self->{'dst'}->{'label'} = undef;
 
+	if ($self->{'is_tty'}) {
+	    if ($self->{'last_is_size'}) {
+		print STDERR "\n";
+		$self->{'last_is_size'} = 0;
+	    }
+	}
 	print STDERR "Could not start new destination volume: $params{error}";
     }
 }
@@ -836,6 +940,12 @@ sub scribe_notif_tape_done {
     step inventory_cb => sub {
 	my ($err, $inventory) = @_;
 	if ($err) {
+	    if ($self->{'is_tty'}) {
+		if ($self->{'last_is_size'}) {
+		    print STDERR "\n";
+		    $self->{'last_is_size'} = 0;
+		}
+	    }
 	    print STDERR "Could not get destination inventory: $err\n";
 	    return $steps->{'done'}->();
 	}
@@ -854,9 +964,21 @@ sub scribe_notif_tape_done {
 	}
 
 	if (!$ie_slot) {
+	    if ($self->{'is_tty'}) {
+		if ($self->{'last_is_size'}) {
+		    print STDERR "\n";
+		    $self->{'last_is_size'} = 0;
+		}
+	    }
 	    print STDERR "No import/export slots available; skipping export\n";
 	    return $steps->{'done'}->();
 	} elsif (!$from_slot) {
+	    if ($self->{'is_tty'}) {
+		if ($self->{'last_is_size'}) {
+		    print STDERR "\n";
+		    $self->{'last_is_size'} = 0;
+		}
+	    }
 	    print STDERR "Could not find the just-written tape; skipping export\n";
 	    return $steps->{'done'}->();
 	} else {
@@ -881,6 +1003,12 @@ sub scribe_notif_tape_done {
     step moved => sub {
 	my ($err) = @_;
 	if ($err) {
+	    if ($self->{'is_tty'}) {
+		if ($self->{'last_is_size'}) {
+		    print STDERR "\n";
+		    $self->{'last_is_size'} = 0;
+		}
+	    }
 	    print STDERR "While exporting just-written tape: $err (ignored)\n";
 	}
 	$steps->{'done'}->();
@@ -937,7 +1065,8 @@ sub usage {
 **NOTE** this interface is under development and will change in future releases!
 
 Usage: amvault [-o configoption...] [-q] [--quiet] [-n] [--dry-run]
-	   [--fulls-only] [--export] [--src-timestamp src-timestamp]
+	   [--fulls-only] [--latest-fulls] [--incrs-only] [--export]
+	   [--src-timestamp src-timestamp] [--exact-match]
 	   --label-template label-template --dst-changer dst-changer
 	   [--autolabel autolabel-arg...]
 	   config
@@ -946,6 +1075,8 @@ Usage: amvault [-o configoption...] [-q] [--quiet] [-n] [--dry-run]
     -o: configuration override (see amanda(8))
     -q: quiet progress messages
     --fulls-only: only copy full (level-0) dumps
+    --latest-fulls: copy the latest full of every dle
+    --incrs-only: only copy incremental (level > 0) dumps
     --export: move completed destination volumes to import/export slots
     --src-timestamp: the timestamp of the Amanda run that should be vaulted
     --label-template: the template to use for new volume labels
@@ -973,6 +1104,9 @@ my @config_overrides_opts;
 my $opt_quiet = 0;
 my $opt_dry_run = 0;
 my $opt_fulls_only = 0;
+my $opt_latest_fulls = 0;
+my $opt_incrs_only = 0;
+my $opt_exact_match = 0;
 my $opt_export = 0;
 my $opt_autolabel = {};
 my $opt_autolabel_seen = 0;
@@ -985,25 +1119,32 @@ sub set_label_template {
 }
 
 sub add_autolabel {
-    my ($opt, $val) = @_;
-    $val = lc($val);
-    $val =~ s/-/_/g;
+    my ($opt, $values) = @_;
+    $values = lc($values);
+    $values =~ s/-/_/g;
 
-    $opt_autolabel_seen = 1;
-    my @ok = qw(other_config non_amanda volume_error empty);
-    for (@ok) {
-	if ($val eq $_) {
-	    $opt_autolabel->{$_} = 1;
-	    return;
-	}
-    }
-    if ($val eq 'any') {
+    my @values = split ' ', $values;
+    my @bad_values;
+    foreach my $val (@values) {
+	$opt_autolabel_seen = 1;
+	my @ok = qw(other_config non_amanda volume_error empty);
 	for (@ok) {
-	    $opt_autolabel->{$_} = 1;
+	    if ($val eq $_) {
+		$opt_autolabel->{$_} = 1;
+		$val = "";
+	    }
 	}
-	return;
+	if ($val eq 'any') {
+	    for (@ok) {
+		$opt_autolabel->{$_} = 1;
+	    }
+	    $val = "";
+	}
+	push @bad_values, $val if $val ne "";
+    }
+    if (@bad_values) {
+	usage("unknown --autolabel value '" . join(',', @bad_values) . "'");
     }
-    usage("unknown --autolabel value '$val'");
 }
 
 debug("Arguments: " . join(' ', @ARGV));
@@ -1016,6 +1157,9 @@ GetOptions(
     'q|quiet' => \$opt_quiet,
     'n|dry-run' => \$opt_dry_run,
     'fulls-only' => \$opt_fulls_only,
+    'latest-fulls' => \$opt_latest_fulls,
+    'incrs-only' => \$opt_incrs_only,
+    'exact-match' => \$opt_exact_match,
     'export' => \$opt_export,
     'label-template=s' => \&set_label_template,
     'autolabel=s' => \&add_autolabel,
@@ -1029,13 +1173,19 @@ $opt_autolabel->{'empty'} = 1 unless $opt_autolabel_seen;
 usage("not enough arguments") unless (@ARGV >= 1);
 
 my $config_name = shift @ARGV;
-my @opt_dumpspecs = parse_dumpspecs(\@ARGV, $CMDLINE_PARSE_DATESTAMP|$CMDLINE_PARSE_LEVEL)
+my $cmd_flags = $CMDLINE_PARSE_DATESTAMP|$CMDLINE_PARSE_LEVEL;
+$cmd_flags |= $CMDLINE_EXACT_MATCH if $opt_exact_match;
+my @opt_dumpspecs = parse_dumpspecs(\@ARGV, $cmd_flags)
     if (@ARGV);
 
 usage("no --label-template given") unless $opt_autolabel->{'template'};
 usage("no --dst-changer given") unless $opt_dst_changer;
 usage("specify something to select the source dumps") unless
-    $opt_src_write_timestamp or $opt_fulls_only or @opt_dumpspecs;
+    $opt_src_write_timestamp or $opt_fulls_only or $opt_latest_fulls or
+    $opt_incrs_only or @opt_dumpspecs;
+
+usage("The following options are incompatible: --fulls-only, --latest-fulls and --incrs-only") if
+      ($opt_fulls_only + $opt_latest_fulls + $opt_incrs_only) > 1;
 
 set_config_overrides($config_overrides);
 config_init($CONFIG_INIT_EXPLICIT_NAME, $config_name);
@@ -1050,6 +1200,14 @@ if ($cfgerr_level >= $CFGERR_WARNINGS) {
 
 Amanda::Util::finish_setup($RUNNING_AS_DUMPUSER);
 
+# and the disklist
+my $diskfile = Amanda::Config::config_dir_relative(getconf($CNF_DISKFILE));
+$cfgerr_level = Amanda::Disklist::read_disklist('filename' => $diskfile);
+if ($cfgerr_level >= $CFGERR_ERRORS) {
+    print STDERR "errors processing disklist\n";
+    exit(1);
+}
+
 my $exit_status;
 my $exit_cb = sub {
     ($exit_status) = @_;
@@ -1066,6 +1224,8 @@ my $vault = Amvault->new(
     opt_dry_run => $opt_dry_run,
     quiet => $opt_quiet,
     fulls_only => $opt_fulls_only,
+    latest_fulls=> $opt_latest_fulls,
+    incrs_only => $opt_incrs_only,
     opt_export => $opt_export,
     config_overrides_opts => \@config_overrides_opts);
 Amanda::MainLoop::call_later(sub { $vault->run($exit_cb) });
