@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -38,8 +38,8 @@
 #include "util.h"
 #include "amxml.h"
 
-static am_host_t *hostlist;
-static netif_t *all_netifs;
+static am_host_t *hostlist = NULL;
+static netif_t *all_netifs = NULL;
 
 /* local functions */
 static char *upcase(char *st);
@@ -58,8 +58,9 @@ read_diskfile(
     char *line = NULL;
 
     /* initialize */
-    hostlist = NULL;
-    lst->head = lst->tail = NULL;
+    if (hostlist == NULL) {
+	lst->head = lst->tail = NULL;
+    }
     line_num = 0;
 
     /* if we already have config errors, then don't bother */
@@ -334,31 +335,29 @@ remove_disk(
 }
 
 void
-free_disklist(
-    disklist_t* dl)
+unload_disklist(void)
 {
-    disk_t    *dp;
+    disk_t    *dp, *dpnext;
     am_host_t *host, *hostnext;
     netif_t *netif, *next_if;
 
-    while (dl->head != NULL) {
-	dp = dequeue_disk(dl);
-	amfree(dp->filename);
-	amfree(dp->name);
-	amfree(dp->hostname);
-	amfree(dp->device);
-	free_sl(dp->exclude_file);
-	free_sl(dp->exclude_list);
-	free_sl(dp->include_file);
-	free_sl(dp->include_list);
-	free(dp);
-    }
-
     for(host=hostlist; host != NULL; host = hostnext) {
 	amfree(host->hostname);
 	am_release_feature_set(host->features);
 	host->features = NULL;
 	hostnext = host->next;
+	for (dp = host->disks; dp != NULL ; dp = dpnext) {
+	    dpnext = dp->hostnext;
+	    amfree(dp->filename);
+	    amfree(dp->name);
+	    amfree(dp->hostname);
+	    amfree(dp->device);
+	    free_sl(dp->exclude_file);
+	    free_sl(dp->exclude_list);
+	    free_sl(dp->include_file);
+	    free_sl(dp->include_list);
+	    free(dp);
+	}
 	amfree(host);
     }
     hostlist=NULL;
@@ -745,7 +744,8 @@ parse_diskline(
     disk->record	     = dumptype_get_record(dtype) != 0;
     disk->skip_incr	     = dumptype_get_skip_incr(dtype) != 0;
     disk->skip_full	     = dumptype_get_skip_full(dtype) != 0;
-    disk->to_holdingdisk     = dumptype_get_to_holdingdisk(dtype);
+    disk->orig_holdingdisk   = dumptype_get_to_holdingdisk(dtype);
+    disk->to_holdingdisk     = disk->orig_holdingdisk;
     disk->kencrypt	     = dumptype_get_kencrypt(dtype) != 0;
     disk->index		     = dumptype_get_index(dtype) != 0; 
 
