@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -116,7 +116,8 @@ typedef struct disk_s {
     int		record;			/* record dump in /etc/dumpdates ? */
     int		skip_incr;		/* incs done externally ? */
     int		skip_full;		/* fulls done externally ? */
-    int		to_holdingdisk;		/* use holding disk ? */
+    int		orig_holdingdisk;	/* original holdingdisk setting */
+    int		to_holdingdisk;		/* holding disk setting */
     int		kencrypt;
     int		index;			/* produce an index ? */
     data_path_t	data_path;		/* defined data-path */
@@ -183,7 +184,7 @@ char *match_disklist(disklist_t *origqp, gboolean exact_match, int sargc,
 		     char **sargv);
 gboolean match_dumpfile(dumpfile_t *file, gboolean exact_match, int sargc,
 			char **sargv);
-void free_disklist(disklist_t *dl);
+void unload_disklist(void);
 
 netif_t *disklist_netifs(void);
 
