@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -575,6 +575,7 @@ dumper_cmd(
     char *cmdline = NULL;
     char number[NUM_STR_SIZE];
     char numberport[NUM_STR_SIZE];
+    char maxdumps[NUM_STR_SIZE];
     char maxwarnings[NUM_STR_SIZE];
     char *o, *oo;
     char *device;
@@ -613,6 +614,7 @@ dumper_cmd(
 	    qname = quote_string(dp->name);
 	    g_snprintf(number, SIZEOF(number), "%d", sched(dp)->level);
 	    g_snprintf(numberport, SIZEOF(numberport), "%d", dumper->output_port);
+	    g_snprintf(maxdumps, SIZEOF(maxdumps), "%d", dp->host->maxdumps);
 	    g_snprintf(maxwarnings, SIZEOF(maxwarnings), "%d", dp->max_warnings);
 	    features = am_feature_to_string(dp->host->features);
 	    if (am_has_feature(dp->host->features, fe_req_xml)) {
@@ -653,6 +655,7 @@ dumper_cmd(
 	    cmdline = vstralloc(cmdstr[cmd],
 			    " ", disk2serial(dp),
 			    " ", numberport,
+			    " ", maxdumps,
 			    " ", dp->host->hostname,
 			    " ", features,
 			    " ", qname,
@@ -977,6 +980,11 @@ update_info_dumper(
 
     level = sched(dp)->level;
 
+    if (origsize == 0 || dumpsize == 0) {
+	g_debug("not updating because origsize or dumpsize is 0");
+	return;
+    }
+
     conf_infofile = config_dir_relative(getconf_str(CNF_INFOFILE));
     if (open_infofile(conf_infofile)) {
 	error(_("could not open info db \"%s\""), conf_infofile);
@@ -1076,6 +1084,11 @@ update_info_taper(
     stats_t *infp;
     int rc;
 
+    if (!label) {
+	log_add(L_ERROR, "update_info_taper without label");
+	return;
+    }
+
     rc = open_infofile(getconf_str(CNF_INFOFILE));
     if(rc) {
 	error(_("could not open infofile %s: %s (%d)"), getconf_str(CNF_INFOFILE),
