@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -104,6 +105,7 @@ static kencrypt_type dumper_kencrypt;
 
 static FILE *errf = NULL;
 static char *hostname = NULL;
+static char *maxdumps = NULL;
 am_feature_t *their_features = NULL;
 static char *diskname = NULL;
 static char *qdiskname = NULL, *b64disk;
@@ -127,6 +129,7 @@ static char *dle_str = NULL;
 static char *errfname = NULL;
 static int   errf_lines = 0;
 static int   max_warnings = 0;
+static char *state_filename = NULL;
 
 static dumpfile_t file;
 
@@ -309,6 +312,13 @@ xml_check_options(
     } else {
 	srvencrypt = ENCRYPT_NONE;
     }
+
+    if (dle->kencrypt) {
+	dumper_kencrypt = KENCRYPT_WILL_DO;
+    } else {
+	dumper_kencrypt = KENCRYPT_NONE;
+    }
+
     free_dle(dle);
     amfree(o);
 }
@@ -452,6 +462,12 @@ main(
 	    }
 	    header_port = (in_port_t)atoi(cmdargs->argv[a++]);
 
+	    if(a >= cmdargs->argc) {
+		error(_("error [dumper PORT-DUMP: not enough args: maxdumps]"));
+		/*NOTREACHED*/
+	    }
+	    maxdumps = newstralloc(maxdumps, cmdargs->argv[a++]);
+
 	    if(a >= cmdargs->argc) {
 		error(_("error [dumper PORT-DUMP: not enough args: hostname]"));
 		/*NOTREACHED*/
@@ -537,6 +553,9 @@ main(
 		error(_("error [dumper PORT-DUMP: not enough args: dataport_list]"));
 	    }
 	    dataport_list = newstralloc(dataport_list, cmdargs->argv[a++]);
+	    if (data_path == DATA_PATH_DIRECTTCP && *dataport_list == '\0') {
+		error(_("error [dumper PORT-DUMP: dataport_list empty for DIRECTTCP]"));
+	    }
 
 	    if(a >= cmdargs->argc) {
 		error(_("error [dumper PORT-DUMP: not enough args: max_warnings]"));
@@ -875,6 +894,24 @@ process_dumpline(
 	    return;
 	}
 
+	if (g_str_equal(tok, "state")) {
+	    FILE *statefile;
+	    tok = strtok(NULL, "");
+	    if (tok) {
+		statefile = fopen(state_filename, "a");
+		if (statefile) {
+		    fprintf(statefile, "%s\n", tok);
+		    fclose(statefile);
+		} else {
+		    g_debug("Can't open statefile '%s': %s", state_filename, strerror(errno));
+		}
+	    } else {
+		g_debug("Invalid state");
+	    }
+	    amfree(buf);
+	    return;
+	}
+
 	if (strcmp(tok, "end") == 0) {
 	    SET(status, GOT_ENDLINE);
 	    break;
@@ -1191,6 +1228,8 @@ do_dump(
     pid_t indexpid = -1;
     char *m;
     int to_unlink = 1;
+    char *shostname;
+    char *sdiskname;
 
     startclock();
 
@@ -1225,6 +1264,14 @@ do_dump(
 	goto failed;
     }
 
+    shostname = sanitise_filename(hostname);
+    sdiskname = sanitise_filename(diskname);
+    state_filename = g_strdup_printf("%s/%s/%s/%s_%d.state",
+		 getconf_str(CNF_INDEXDIR), shostname,
+		 sdiskname, dumper_timestamp, level);
+    amfree(shostname);
+    amfree(sdiskname);
+
     if (streams[INDEXFD].fd != NULL) {
 	indexfile_real = getindexfname(hostname, diskname, dumper_timestamp, level);
 	indexfile_tmp = stralloc2(indexfile_real, ".tmp");
@@ -1315,7 +1362,7 @@ do_dump(
     }
 
     /* copy the header in a file on the index dir */
-    {
+    if (ISSET(status, HEADER_DONE)) {
 	FILE *a;
 	char *s;
 	char *f = getheaderfname(hostname, diskname, dumper_timestamp, level);
@@ -1437,6 +1484,7 @@ do_dump(
     if (data_path == DATA_PATH_AMANDA)
 	aclose(db->fd);
 
+    amfree(state_filename);
     amfree(errstr);
     dumpfile_free_data(&file);
 
@@ -1566,19 +1614,18 @@ read_mesgfd(
     default:
 	assert(buf != NULL);
 	add_msg_data(buf, (size_t)size);
-	security_stream_read(streams[MESGFD].fd, read_mesgfd, cookie);
 	break;
     }
 
     if (ISSET(status, GOT_INFO_ENDLINE) && !ISSET(status, HEADER_DONE)) {
 	/* Use the first in the dataport_list */
 	in_port_t data_port;
-	char *data_host = dataport_list;
+	char *data_host = g_strdup(dataport_list);
 	char *s;
 
-	s = strchr(dataport_list, ',');
+	s = strchr(data_host, ',');
 	if (s) *s = '\0';  /* use first data_port */
-	s = strrchr(dataport_list, ':');
+	s = strrchr(data_host, ':');
 	*s = '\0';
 	s++;
 	data_port = atoi(s);
@@ -1587,14 +1634,59 @@ read_mesgfd(
 	/* time to do the header */
 	finish_tapeheader(&file);
 	if (write_tapeheader(db->fd, &file)) {
-	    errstr = newvstrallocf(errstr, _("write_tapeheader: %s"), 
+	    errstr = newvstrallocf(errstr, _("write_tapeheader: %s"),
 				  strerror(errno));
 	    dump_result = 2;
+	    amfree(data_host);
 	    stop_dump();
 	    return;
 	}
 	aclose(db->fd);
 	if (data_path == DATA_PATH_AMANDA) {
+	    /* do indirecttcp */
+	    if (g_str_equal(data_host,"255.255.255.255")) {
+		char buffer[32770];
+		char *s;
+		int size;
+
+		g_debug(_("Sendingd indirect data output stream: %s:%d\n"), data_host, data_port);
+		db->fd = stream_client("localhost", data_port,
+				       STREAM_BUFSIZE, 0, NULL, 0);
+		if (db->fd == -1) {
+                    g_free(errstr);
+                    errstr = g_strdup_printf(_("Can't open indirect data output stream: %s"),
+                                         strerror(errno));
+		    dump_result = 2;
+		    amfree(data_host);
+		    stop_dump();
+		    return;
+		}
+	        size = full_read(db->fd, buffer, 32768);
+		if (size <= 0) {
+		    g_debug("Failed to read from indirect-direct-tcp port: %s",
+			    strerror(errno));
+		    close(db->fd);
+		    dump_result = 2;
+		    amfree(data_host);
+		    stop_dump();
+		    return;
+		}
+		aclose(db->fd);
+		buffer[size++] = ' ';
+                buffer[size] = '\0';
+                if ((s = strchr(buffer, ':')) == NULL) {
+		    g_debug("Failed to parse indirect data output stream: %s", buffer);
+		    dump_result = 2;
+		    amfree(data_host);
+		    stop_dump();
+		    return;
+                }
+		*s++ = '\0';
+		amfree(data_host);
+		data_host = g_strdup(buffer);
+		data_port = atoi(s);
+	    }
+
 	    g_debug(_("Sending data to %s:%d\n"), data_host, data_port);
 	    db->fd = stream_client(data_host, data_port,
 				   STREAM_BUFSIZE, 0, NULL, 0);
@@ -1603,10 +1695,12 @@ read_mesgfd(
 				       _("Can't open data output stream: %s"),
 				       strerror(errno));
 		dump_result = 2;
+		amfree(data_host);
 		stop_dump();
 		return;
 	    }
 	}
+	amfree(data_host);
 
 	dumpsize += (off_t)DISK_BLOCK_KB;
 	headersize += (off_t)DISK_BLOCK_KB;
@@ -1629,8 +1723,10 @@ read_mesgfd(
 		return;
 	    }
 	}
-	security_stream_read(streams[DATAFD].fd, read_datafd, db);
-	set_datafd = 1;
+	if (data_path == DATA_PATH_AMANDA) {
+	    security_stream_read(streams[DATAFD].fd, read_datafd, db);
+	    set_datafd = 1;
+	}
     }
 
     /*
@@ -1703,8 +1799,6 @@ read_datafd(
      * Reset the timeout for future reads
      */
     timeout(conf_dtimeout);
-
-    security_stream_read(streams[DATAFD].fd, read_datafd, cookie);
 }
 
 /*
@@ -1757,7 +1851,6 @@ read_indexfd(
 	    log_add(L_INFO, _("Index corrupted for %s:%s"), hostname, qdiskname);
 	}
     }
-    security_stream_read(streams[INDEXFD].fd, read_indexfd, cookie);
 }
 
 static void
@@ -1769,8 +1862,6 @@ handle_filter_stderr(
     char     *b, *p;
     gint64    len;
 
-    event_release(filter->event);
-
     if (filter->buffer == NULL) {
 	/* allocate initial buffer */
 	filter->buffer = g_malloc(2048);
@@ -1793,6 +1884,7 @@ handle_filter_stderr(
 			     filter->allocated_size - filter->first - filter->size - 2);
 
     if (nread <= 0) {
+	event_release(filter->event);
 	aclose(filter->fd);
 	if (filter->size > 0 && filter->buffer[filter->first + filter->size - 1] != '\n') {
 	    /* Add a '\n' at end of buffer */
@@ -1823,9 +1915,6 @@ handle_filter_stderr(
     if (nread <= 0) {
 	g_free(filter->buffer);
 	g_free(filter);
-    } else {
-	filter->event = event_register((event_id_t)filter->fd, EV_READFD,
-				       handle_filter_stderr, filter);
     }
 }
 
@@ -1833,25 +1922,33 @@ handle_filter_stderr(
  * Startup a timeout in the event handler.  If the arg is 0,
  * then remove the timeout.
  */
+static event_handle_t *ev_timeout = NULL;
+static time_t timeout_time;
+
 static void
 timeout(
     time_t seconds)
 {
-    static event_handle_t *ev_timeout = NULL;
+    timeout_time = time(NULL) + seconds;
 
     /*
-     * First, remove a timeout if one is active.
+     * remove a timeout if seconds is 0
      */
-    if (ev_timeout != NULL) {
-	event_release(ev_timeout);
-	ev_timeout = NULL;
+    if (seconds == 0) {
+	if (ev_timeout != NULL) {
+	    event_release(ev_timeout);
+	    ev_timeout = NULL;
+	}
+	return;
     }
 
     /*
-     * Now, schedule a new one if 'seconds' is greater than 0
+     * schedule a timeout if it not already scheduled
      */
-    if (seconds > 0)
-	ev_timeout = event_register((event_id_t)seconds, EV_TIME, timeout_callback, NULL);
+    if (ev_timeout == NULL) {
+	ev_timeout = event_register((event_id_t)seconds, EV_TIME,
+				     timeout_callback, NULL);
+    }
 }
 
 /*
@@ -1862,8 +1959,20 @@ static void
 timeout_callback(
     void *	unused)
 {
+    time_t now = time(NULL);
     (void)unused;	/* Quiet unused parameter warning */
 
+    if (ev_timeout != NULL) {
+	event_release(ev_timeout);
+	ev_timeout = NULL;
+    }
+
+    if (timeout_time > now) { /* not a data timeout yet */
+	ev_timeout = event_register((event_id_t)(timeout_time-now), EV_TIME,
+				    timeout_callback, NULL);
+	return;
+    }
+
     assert(unused == NULL);
     errstr = newstralloc(errstr, _("data timeout"));
     dump_result = 2;
@@ -1883,8 +1992,15 @@ stop_dump(void)
     /* Check if I have a pending ABORT command */
     cmdargs = get_pending_cmd();
     if (cmdargs) {
+	if (cmdargs->cmd == QUIT) {
+	    g_debug("Got unexpected QUIT");
+	    log_add(L_FAIL, "%s %s %s %d [Killed while dumping]",
+		    hostname, qdiskname, dumper_timestamp, level);
+	    exit(1);
+	}
 	if (cmdargs->cmd != ABORT) {
-	    error(_("beurk %d"), cmdargs->cmd);
+	    g_debug("Expected an ABORT command");
+	    exit(1);
 	}
 	amfree(errstr);
 	errstr = stralloc(cmdargs->argv[1]);
@@ -2354,6 +2470,7 @@ startup_dump(
     const security_driver_t *secdrv;
     char *application_api;
     int has_features;
+    int has_maxdumps;
     int has_hostname;
     int has_device;
     int has_config;
@@ -2367,6 +2484,7 @@ startup_dump(
 
     has_features = am_has_feature(their_features, fe_req_options_features);
     has_hostname = am_has_feature(their_features, fe_req_options_hostname);
+    has_maxdumps = am_has_feature(their_features, fe_req_options_maxdumps);
     has_config   = am_has_feature(their_features, fe_req_options_config);
     has_device   = am_has_feature(their_features, fe_sendbackup_req_device);
 
@@ -2389,6 +2507,9 @@ startup_dump(
 		    has_features ? "features=" : "",
 		    has_features ? our_feature_string : "",
 		    has_features ? ";" : "",
+		    has_maxdumps ? "maxdumps=" : "",
+		    has_maxdumps ? maxdumps : "",
+		    has_maxdumps ? ";" : "",
 		    has_hostname ? "hostname=" : "",
 		    has_hostname ? hostname : "",
 		    has_hostname ? ";" : "",
