@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1999 University of Maryland at College Park
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -98,6 +99,7 @@ typedef struct est_s {
     one_est_t *dump_est;
     one_est_t *degr_est;
     one_est_t  estimate[MAX_LEVELS];
+    gboolean   allow_server;
     int last_level;
     gint64 last_lev0size;
     int next_level0;
@@ -193,6 +195,7 @@ main(
     int exit_status = EXIT_SUCCESS;
     gboolean no_taper = FALSE;
     gboolean from_client = FALSE;
+    gboolean exact_match = FALSE;
 
     if (argc > 1 && argv && argv[1] && g_str_equal(argv[1], "--version")) {
 	printf("planner-%s\n", VERSION);
@@ -276,6 +279,11 @@ main(
 	from_client = TRUE;
 	diskarg_offset += 1;
     }
+    if (argc - diskarg_offset > 0 && g_str_equal(argv[diskarg_offset],
+                                                 "--exact_match")) {
+	exact_match = TRUE;
+	diskarg_offset += 1;
+    }
 
 
     run_server_global_scripts(EXECUTE_ON_PRE_ESTIMATE, get_config_name());
@@ -348,7 +356,7 @@ main(
     g_fprintf(stderr, _("%s: timestamp %s\n"),
 		    get_pname(), planner_timestamp);
 
-    errstr = match_disklist(&origq, argc-diskarg_offset,
+    errstr = match_disklist(&origq, exact_match, argc-diskarg_offset,
 				    argv+diskarg_offset);
     if (errstr) {
 	g_fprintf(stderr,"%s",errstr);
@@ -446,12 +454,16 @@ main(
 	GSList *holding_list, *holding_file;
 	char *qdisk, *qhname;
 
+	holding_cleanup(NULL, stderr);
+
 	/* get *all* flushable files in holding, without checking against
 	 * the disklist (which may not contain some of the dumps) */
 	holding_list = holding_get_files_for_flush(NULL);
 	for(holding_file=holding_list; holding_file != NULL;
 				       holding_file = holding_file->next) {
-	    holding_file_get_dumpfile((char *)holding_file->data, &file);
+	    if (!holding_file_get_dumpfile((char *)holding_file->data, &file)) {
+		continue;
+	    }
 
 	    if (holding_file_size((char *)holding_file->data, 1) <= 0) {
 		log_add(L_INFO, "%s: removing file with no data.",
@@ -463,7 +475,7 @@ main(
 
 	    /* see if this matches the command-line arguments */
 	    if (conf_autoflush == 1 &&
-		!match_dumpfile(&file, argc-diskarg_offset,
+		!match_dumpfile(&file, exact_match, argc-diskarg_offset,
 				       argv+diskarg_offset)) {
 		continue;
 	    }
@@ -863,6 +875,25 @@ setup_estimate(
 		    dp->host->hostname, qname);
 	}
     }
+    else if (ISSET(info->command, FORCE_LEVEL_1)) {
+	/* force a level 1 */
+	if (dp->strategy == DS_NOINC) {
+	    log_add(L_WARNING,
+		    _("Cannot force level 1 dump of %s:%s with no-incr option."),
+		    dp->host->hostname, qname);
+
+	    /* clear force-level-1 command */
+	    CLR(info->command, FORCE_LEVEL_1);
+	    ep->last_level = last_level(info);
+	    ep->next_level0 = next_level0(dp, info);
+	} else {
+	    ep->degr_mesg = "";
+	    ep->last_level = 0;
+	    ep->next_level0 = 0;
+	    log_add(L_INFO, _("Forcing level 1 of %s:%s as directed."),
+		    dp->host->hostname, qname);
+	}
+    }
     else if(dp->strategy == DS_NOFULL) {
 	/* force estimate of level 1 */
 	ep->last_level = 1;
@@ -1018,6 +1049,7 @@ setup_estimate(
 
     if (dp->strategy == DS_NOINC ||
 	(!dp->skip_full &&
+	 !ISSET(info->command, FORCE_LEVEL_1) &&
 	 (!ISSET(info->command, FORCE_BUMP) ||
 	  dp->skip_incr ||
 	  ep->last_level == -1))) {
@@ -1027,11 +1059,15 @@ setup_estimate(
 		    dp->host->hostname, qname);
 	}
 	switch (dp->strategy) {
-	case DS_STANDARD: 
+	case DS_STANDARD:
 	case DS_NOINC:
 	    askfor(ep, i++, 0, info);
 	    if (ep->last_level == -1)
-		ep->degr_mesg = _("Skipping: new disk can't be dumped in degraded mode");
+		if (ISSET(info->command, FORCE_FULL)) {
+		    ep->degr_mesg = _("Skipping: Can't bump a 'force' dle in degraded mode");
+		} else {
+		    ep->degr_mesg = _("Skipping: new disk can't be dumped in degraded mode");
+		}
 	    else
 		ep->degr_mesg = _("Skipping: strategy NOINC can't be dumped in degraded mode");
 	    if(dp->skip_full) {
@@ -1044,7 +1080,7 @@ setup_estimate(
 		 _("Ignoring FORCE_BUMP for %s:%s because the strategy is NOINC."),
 			dp->host->hostname, qname);
 	    }
-	    
+
 	    break;
 
 	case DS_NOFULL:
@@ -1071,7 +1107,10 @@ setup_estimate(
 
 	    curr_level = ep->last_level;
 
-	    if (ISSET(info->command, FORCE_NO_BUMP)) {
+	    if (ISSET(info->command, FORCE_LEVEL_1)) {
+		askfor(ep, i++, 1, info);
+		ep->degr_mesg = _("Skipping: force-level-1 disk can't be dumped in degraded mode");
+	    } else if (ISSET(info->command, FORCE_NO_BUMP)) {
 		if(curr_level > 0) { /* level 0 already asked for */
 		    askfor(ep, i++, curr_level, info);
 		}
@@ -1521,7 +1560,8 @@ static void getsize(
 		if (estimate == ES_SERVER)
 		    break;
 	    }
-	    if (estimate == ES_SERVER) {
+	    est(dp)->allow_server = estimate == ES_SERVER;
+	    if (est(dp)->allow_server) {
 		info_t info;
 		nb_server++;
 		get_info(dp->host->hostname, dp->name, &info);
@@ -1841,7 +1881,7 @@ static void handle_result(
 
     if (pkt == NULL) {
 	if (strcmp(security_geterror(sech), "timeout waiting for REP") == 0) {
-	    errbuf = vstrallocf("Some estimate timeout on %s, using server estimate if possible", hostp->hostname);
+	    errbuf = vstrallocf("Some estimate timeout on %s", hostp->hostname);
 	} else {
 	    errbuf = vstrallocf(_("Request to %s failed: %s"),
 			hostp->hostname, security_geterror(sech));
@@ -2139,16 +2179,28 @@ static void handle_result(
     i = 0;
     for(dp = hostp->disks; dp != NULL; dp = dp->hostnext) {
 	if (dp->todo) {
-	    if(est(dp)->state == DISK_ACTIVE) {
+	    if (est(dp)->state == DISK_ACTIVE ||
+		est(dp)->state == DISK_PARTIALY_DONE) {
+		char *tt;
 		qname = quote_string(dp->name);
+		if (est(dp)->state == DISK_ACTIVE) {
+		    remove_disk(&waitq, dp);
+		} else if(est(dp)->state == DISK_PARTIALY_DONE) {
+		    remove_disk(&pestq, dp);
+		}
 		est(dp)->state = DISK_DONE;
-		remove_disk(&waitq, dp);
-		enqueue_disk(&failq, dp);
+		if (est(dp)->allow_server) {
+		    enqueue_disk(&estq, dp);
+		    tt = ", using server estimate";
+		} else {
+		    enqueue_disk(&failq, dp);
+		    tt = "";
+		}
 		i++;
 
-		est(dp)->errstr = stralloc(errbuf);
+		est(dp)->errstr = g_strdup_printf("%s%s", errbuf, tt);
 		g_fprintf(stderr, _("error result for host %s disk %s: %s\n"),
-			  dp->host->hostname, qname, errbuf);
+			  dp->host->hostname, qname, est(dp)->errstr);
 		amfree(qname);
 	    }
 	}
@@ -2159,7 +2211,25 @@ static void handle_result(
 	 * reported.
 	 */
 	log_add(L_ERROR, "%s", errbuf);
+	for(dp = hostp->disks; dp != NULL; dp = dp->hostnext) {
+	    if (dp->todo) {
+		qname = quote_string(dp->name);
+		if(est(dp)->state == DISK_ACTIVE) {
+		    remove_disk(&waitq, dp);
+		} else if(est(dp)->state == DISK_PARTIALY_DONE) {
+		    remove_disk(&pestq, dp);
+		}
+		est(dp)->state = DISK_DONE;
+		enqueue_disk(&failq, dp);
+
+		est(dp)->errstr = g_strdup(errbuf);
+		g_fprintf(stderr, _("error result for host %s disk %s: %s\n"),
+			  dp->host->hostname, qname, errbuf);
+		amfree(qname);
+	    }
+	}
     }
+
     hostp->up = HOST_DONE;
     amfree(errbuf);
     /* try to clean up any defunct processes, since Amanda doesn't wait() for
